package com.mowahs.mwcf.framework;

import java.io.Serializable;
import java.util.Vector;
import java.sql.*;
import java.util.Hashtable;
import java.math.BigDecimal;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionError;

/**
 * This class represents a characteristic in the MWCF framework. A characteristic belongs to one
 * group in the MWCF framework. This class handles all logic tied to characteristic creation,
 * storage and deletion. In addition, the class handles the logic involved in characterising a task.
 * @author Siv Oma Rogdar
 */
public class CharacteristicBean implements Serializable {
	
	private int characteristicID;
	private String name;
	private String description;
	// The characterisation done on this characteristic
	private int score;
	/*
	 * A Vector storing the possible values and their belonging descriptions that can be applied to this characteristic.
	 */
	private Vector scale;
	
	/**
	 * A Vector storing the possible values of this characteristic
	 */
	private Vector values;
	
	/**
	 * A Vector storing the descriptions to the values of this characteristic
	 */
	private Vector descriptions;
	
	//The Vector of all existing characteristics
	private Vector characteristics;
	
	//The computed average of this characteristic for a scenario
	private float scenarioAverage;
	
	public CharacteristicBean() 
	{
		
	}
	
	
	/**
	 * Gets the id representing this characteristic in the database
	 * @return int
	 */
	public int getCharacteristicID() 
	{
		return this.characteristicID;
	}
	
	/**
	 * Sets the id of this characteristic (id is used to represent this characteristic in the database)
	 * @param characteristicID
	 */
	public void setCharacteristicID(int id) 
	{
		this.characteristicID = id;
	}
	
	/**
	 * Gets the name of this characteristic
	 * @return java.lang.String
	 */
	public String getName() 
	{
		return this.name;
	}
	
	/**
	 * Sets the name of this characteristic
	 * @param name
	 */
	public void setName(String name) 
	{
		this.name = name;
	}
	
	/**
	 * Gets the description of this characteristic
	 * @return java.lang.String
	 */
	public String getDescription() 
	{
		return this.description;
	}
	
	/**
	 * Sets the description of this characteristic
	 * @param description
	 */
	public void setDescription(String description) 
	{
		this.description = description;
	}
	
	/**
	 * Gets the Vector of the values and descriptions of values of this characteristic
	 * @return java.util.Vector
	 */
	public Vector getScale() {
		return this.scale;
	}
	
	/**
	 * Sets the Vector of values and descriptions of values of this characteristic
	 * @param java.util.Vector
	 */
	public void setScale(Vector scale) {
		this.scale = scale;
	}
	
	/**
	 * Gets the values this characteristic can get
	 * @return java.util.Vector
	 */
	public Vector getValues() 
	{
		return this.values;
	}
	
	/**
	 * Sets the values this characteristic can get
	 */
	public void setValue(Vector values) 
	{
		this.values = values;
	}
	
	/**
	 * Gets the descriptions of the values this characteristic can get
	 * @return java.util.Vector
	 */
	public Vector getDescriptions() 
	{
		return this.descriptions;
	}
	
	/**
	 * Sets the descriptions of the values this characteristic can get
	 */
	public void setDescriptions(Vector descriptions) 
	{
		this.descriptions = descriptions;
	}
	
	/**
	 * Gets the score on this characteristic for a particular task
	 * @return int The characterisation score
	 */
	public int getScore() {
	
		return this.score;
	
	}
	
	/**
	 * Sets the score on this characteristic for a particular task
	 */
	public void setScore(int score) {
		
		this.score = score;
		
	}
	
	/**
	 * Sets the average score of this characteristic of a particular scenario
	 * @param score The average score
	 */
	 public void setScenarioAverage(float score) {
	 	this.scenarioAverage = score;
	}
	
	/**
	 * Gets the average score of this characteristic of a particular scenario
	 * @return The average score as a float with two decimals
	 */
	public float getScenarioAverage() {
		return this.scenarioAverage;
	}
	
	
	/**
	 * Sets the Vector of CharacteristicBean objects
	 * @param characs A Vector of all existing CharacteristicBean objects 
	 */
	public void setCharacteristics(Vector characs) {
		this.characteristics = characs;
	}
	
	/**
	 * Gets the Vector variable of this characteristic that contains characteristic objects of all existing characteristics in the MWCF framework
	 * @return A Vector of CharacteristicBean objects
	 */
	public Vector getCharacteristics() {
		return this.characteristics;
	}
	
	/**
	 * Retrieves all the characteristics in the database and sets the Vector of CharacteristicBean objects, which may be retrieved by the getCharacteristics()-method.
	 * @param conn A connection with the database
	 * @param errors To store errors in
	 */
	public void getAllCharacteristicsFromDB(Connection conn, ActionErrors errors) {
		
		try {
			Statement stmt = conn.createStatement();
			ResultSet rs = stmt.executeQuery("select * from T_CHARACTERISTIC");
			Vector characs = new Vector();
			
			while(rs.next()) {
				CharacteristicBean charac = new CharacteristicBean();
	        	charac.setCharacteristicID(rs.getInt("CHARACTERISTIC_ID"));
	        	charac.setName(rs.getString("NAME"));
	        	charac.setDescription(rs.getString("DESCRIPTION"));
	        	characs.add(charac);
	        	} 
        	this.setCharacteristics(characs);   
		
        } catch(SQLException sqle) {
	    	errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("error.database"));
   		}	
	}
	
	public void getCharacteristicsFromDB(Connection conn, ActionErrors errors, Vector characteristicIDs) {
		
		try {
			int j = characteristicIDs.size();
			Vector characs = new Vector();
			for(int i = 0; i < j; i++) {
				int ID = ((Integer)characteristicIDs.elementAt(i)).intValue();
				Statement stmt = conn.createStatement();
				ResultSet rs = stmt.executeQuery("select * from T_CHARACTERISTIC where CHARACTERISTIC_ID="+ID);
				
			
				while(rs.next()) {
					CharacteristicBean charac = new CharacteristicBean();
	        		charac.setCharacteristicID(rs.getInt("CHARACTERISTIC_ID"));
	        		charac.setName(rs.getString("NAME"));
	        		charac.setDescription(rs.getString("DESCRIPTION"));
	        		characs.add(charac);
	        		}
	        }
        	this.setCharacteristics(characs);   
		
        } catch(SQLException sqle) {
	    	errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("error.database"));
   		}	
	}
	 
	/**
	 * Removes the characteristic completely from the database.
	 * Removes all stored records of this characteristic from all tables affected
	 * by this characteristic in the database.
	 * @param conn A connection with the database
	 * @param errors
	 * @param characID The id of this characteristic in the database.
	 */
	public void removeCharacteristicFromDB(Connection conn, ActionErrors errors, int characID) 
	{
		int i = -1;
		try {
   			Statement stmt = conn.createStatement();
   			stmt.executeUpdate("delete from T_TASK_CHAR where CHARACTERISTIC_ID = "+characID);
						
			stmt.executeUpdate("delete from T_INVOLVEMENT where CHARACTERISTIC_ID = "+characID);
			
			stmt.executeUpdate("delete from T_CHAR_VALUE where CHARACTERISTIC_ID = "+characID);
			
			stmt.executeUpdate("delete from T_CHARACTERISTIC where CHARACTERISTIC_ID = "+characID);
			
        } catch(SQLException sqle) {
	    	errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("error.database"));
   		}	
	}
	
	
	/**
	 * Get the characterisation done, if any, of this task and this characteristic from the database.
	 * @return boolean Returns true if there is saved a characterisation on this task (the characterisation value 
	 * is saved in the property score on this CharacteristicBean), and false if no characterisation has been done. 
	 *
	 * @param conn A connection with the database
	 * @param errors
	 * @param int The task id to get the characterisation on
	 */
	public boolean getCharacterisationFromDB(Connection conn, ActionErrors errors, int taskID) {
		
		boolean characterised = false;
		try {
   			PreparedStatement stmt = conn.prepareStatement("select VALUE from T_TASK_CHAR where TASK_ID = ? and CHARACTERISTIC_ID = ?");
	       	stmt.setInt(1, taskID);
	       	stmt.setInt(2, this.characteristicID);
	       	ResultSet rs = stmt.executeQuery();
			if (rs.next()) {
				this.setScore(rs.getInt("VALUE"));
				characterised = true;
			}
			
   		} catch(SQLException sqle) {
	    	errors.add("error.database", new ActionError("error.database"));
   		}		
   		return characterised;	
	}
	
	
	/**
	 * Saves a new characteristic in the database.
	 * Requires that the properties name and description of this characteristic have been set. 
	 * Optionally, the property scale may also have been set and will 
	 * then be stored in the database. 
	 * 
	 * @param conn A connection with the database
	 * @param errors An ActionErrors object to store occurring errors in
	 * @param groupID The id of the group that this characteristic belongs to
	 */
	public void saveCharacteristicInDB(Connection conn, ActionErrors errors, int groupID) {
		
		try {
   			Statement stmt = conn.createStatement();
   			int j = stmt.executeUpdate("insert into T_CHARACTERISTIC (NAME, DESCRIPTION, GROUP_ID) values (\""+this.name+"\", \""+this.description+"\", "+groupID+")");
	       
	       	if(j==1) {
				ResultSet rs = stmt.executeQuery("select LAST_INSERT_ID() as lastid");
				if(rs.next()) {
	            	this.setCharacteristicID(rs.getInt("lastid"));
            		if(this.scale != null && !(this.scale.isEmpty())) {
            			PreparedStatement stmtValues = conn.prepareStatement("insert into T_CHAR_VALUE (CHARACTERISTIC_ID, VALUE, DESCRIPTION) values (?, ?, ?)");
            			
            			for(int i=0; i < this.scale.size(); i+=2) {
	            			stmtValues.setInt(1,this.characteristicID);
	       					stmtValues.setInt(2,((Integer)scale.elementAt(i)).intValue());
	       					stmtValues.setString(3,""+scale.elementAt(i+1));
	       					stmtValues.execute();
            			}
            		}
            	}
        	} else {
        		errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("error.database"));
        	}	    
   		} catch(SQLException sqle) {
	    	errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("error.database"));
   		}		
		
	}
	
	/**
	 * Saves the characterisation of this characteristic in the database. 
	 * This method takes the value of the characterisation and the task id as input.
	 * @param conn A connection with the database
	 * @param errors An ActionErrors object to store occurring errors in
	 * @param value The int of the characterisation
	 * @param taskID The id of the task that is being characterised
	 */
	public void saveCharacterisationInDB(Connection conn, ActionErrors errors, int value, int taskID) {
		
		try {
   			PreparedStatement stmt = conn.prepareStatement("insert into T_TASK_CHAR (TASK_ID, CHARACTERISTIC_ID, VALUE) values (?, ?, ?)");
	       	stmt.setInt(1,taskID);
	       	stmt.setInt(2, this.characteristicID);
	       	stmt.setInt(3, value);
	       	int bo = stmt.executeUpdate();
	       	
	       	if(bo != 1)
	       		errors.add("error.database", new ActionError("error.database"));
   		
   		} catch(SQLException sqle) {
	    	errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("database.could.not.connect"));
   		}	
		
	}
	
	/**
	 * Updates a task's characterisation of this characteristic in the database.
	 * This method takes the value of the characterisation and the task id as input.
	 * If there is no characterisation stored on this characteristic and this task in the 
	 * database, this method will insert a new record of the characterisation (might happen if
	 * ie. a change of the framework has taken place).
	 * @param conn A connection with the database
	 * @param errors An ActionErrors object to store occurring errors in
	 * @param value The int of the characterisation
	 * @param taskID The id of the task that is being characterised
	 */
	public void updateCharacterisationInDB(Connection conn, ActionErrors errors, int value, int taskID) {
		
		try {
   			Statement stmt = conn.createStatement();
			int bo = stmt.executeUpdate("update T_TASK_CHAR set VALUE = "+value+" where TASK_ID = "+taskID+" and CHARACTERISTIC_ID = "+this.characteristicID);
			System.out.println("bo: "+bo);
	       	if(bo == 0) {
	       		stmt.execute("insert into T_TASK_CHAR (VALUE, TASK_ID, CHARACTERISTIC_ID) values ("+value+", "+taskID+", "+this.characteristicID+")");
	       	}
   		} catch(SQLException sqle) {
	    	errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("database.could.not.connect"));
   		}	
		
	}
	
	/**
	 * Gets the scale values and descriptions of this characteristic from the database.
	 * Sets the Vector of scales and values, which can be retrieved by the getScale() method.
	 * This method also sets two independent Vectors, one of Integer values (can be retrieved by the getValues() method) 
	 * and another of String descriptions (retrieved by the getDescriptions() method). 
	 * @param conn A connection with the database
	 * @param errors An ActionErrors object to store occurring errors in
	 */
	public void getScalesFromDB(Connection conn, ActionErrors errors) {
	
		try {
   			PreparedStatement stmt = conn.prepareStatement("select VALUE, DESCRIPTION from T_CHAR_VALUE where CHARACTERISTIC_ID = ?");
			stmt.setInt(1, this.characteristicID);
			ResultSet rs = stmt.executeQuery();
			
			Vector valDes = new Vector();
			this.values = new Vector();
			this.descriptions = new Vector();
			while(rs.next()) {
				Integer temp = new Integer(rs.getInt("VALUE"));				
				valDes.add(temp);
				this.values.add(temp);
				valDes.add(rs.getString("DESCRIPTION"));
				this.descriptions.add(rs.getString("DESCRIPTION"));
			}
			this.scale = valDes;
			
   		} catch(SQLException sqle) {
	    	errors.add("error.database", new ActionError("error.database"));
   		}	
		
	}
	
	/**
	 * Updates the name, description and scale of this characteristic in the database.
	 * @param conn A connection with the database
	 * @param errors An ActionErrors object to store occurring errors in
	 */	
	public void updateCharacteristicInDB(Connection conn, ActionErrors errors) {
		
		try {
   			Statement stmt = conn.createStatement();
			//Clears the scale values first
			stmt.executeUpdate("delete from T_CHAR_VALUE where CHARACTERISTIC_ID = "+this.characteristicID);
			
			//Insert new scale values
			if(this.scale != null && !(this.scale.isEmpty())) {
            	for(int i=0; i < this.scale.size(); i+=2) {
	            	stmt.execute("insert into T_CHAR_VALUE (CHARACTERISTIC_ID, VALUE, DESCRIPTION) values ("+this.characteristicID+", "+((Integer)scale.elementAt(i)).intValue()+", \""+scale.elementAt(i+1)+"\")");
	       		}
            }		
			int i = stmt.executeUpdate("update T_CHARACTERISTIC set NAME = \""+this.name+"\", DESCRIPTION = \""+this.description+"\" where CHARACTERISTIC_ID = "+this.characteristicID);
			if(i != 1){
	       		errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("error.database"));
			}
        } catch(SQLException sqle) {
	    	errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("sql.exception"));
   		}	
   		
	}
	
	/**
	 * Calculates the average score on this characteristic for a given scenario.
	 * The score calculated may be retrieved by the getScenarioAverage() method of this characteristic
	 * @param scenarioid The id of the scenario whose average score shall be calculated
	 * @param conn A connection with the database
	 * @param errors An ActionErrors object to store any occurring errors in
	 */
	public void calculateScenarioAverage(Connection conn, ActionErrors errors, int scenarioid) {
		
		try {
   			//Get tasks in scenario and their weights
   			Statement stmt = conn.createStatement();
   			ResultSet rs = stmt.executeQuery("select T_TASK.TASK_ID, T_TASK.WEIGHT from T_TASK, T_ROLE where T_TASK.ROLE_ID = T_ROLE.ROLE_ID and T_ROLE.SCENARIO_ID = "+scenarioid);
   			
   			int taskweight = 0;
   			int tempscore = 0;
   			int weightsum = 0;
   			//Get the task values on this characteristic
   			while(rs.next()) {
   				taskweight = rs.getInt(2);
   		  		Statement stm = conn.createStatement();
   				ResultSet res = stm.executeQuery("select VALUE from T_TASK_CHAR where TASK_ID = "+rs.getInt(1)+" and CHARACTERISTIC_ID = "+this.characteristicID);
   				if(res.next()) {
   					tempscore = tempscore + taskweight*(res.getInt(1)*1000);
   					weightsum = weightsum + taskweight;	
   				}		
   			}
   			if(weightsum > 0 && tempscore > 0){
   				tempscore = tempscore/weightsum;
   			}
   			BigDecimal bgtemp = new BigDecimal((double)tempscore);
   			bgtemp = bgtemp.divide(new BigDecimal(1000), 2, BigDecimal.ROUND_HALF_UP);
   			this.setScenarioAverage(bgtemp.floatValue());
			   		
   		} catch(SQLException sqle) {
	    	errors.add("error.database", new ActionError("error.database"));
   		}		
		
	}
	
	/**
	 * Stores a rule element of a characteristic in the database
	 * @param characteristic_id The id of the characteristic in the database
	 * @param value The value to do the comparison of this characteristic to
	 * @param symbol This may be a String equal to one of the following: "==", "<=", "<", ">", ">=" or "!=".
	 * @return The id of the newly stored rule element in the database
	 
	public int saveRuleElementInDB(Connection conn, ActionErrors errors, int characteristic_id, int value, String symbol) 
	{
		int ruleElementId = 0;
		try {	
			Statement stmt = conn.createStatement();
			int rows = stmt.executeUpdate("insert into T_RULE_ELEMENT (CHARACTERISTIC_ID, VALUE, SYMBOL) values ("+characteristic_id+", "+value+", \""+symbol+"\")");
			if(rows == 1) {
				ResultSet rs = stmt.executeQuery("select LAST_INSERT_ID() as last_insert_id");
				if(rs.next())
					 ruleElementId = rs.getInt(1);
			}
			
		} catch(SQLException sqle) {
		   	errors.add("error.database", new ActionError("error.database"));
	   	}		
	   	
	   	return ruleElementId;
	}
	 */
	 
	/**
	 * Saves a rule of characterisation in the database
	 * @param ruleElements The Vector must contain an iteration of the following elements in that exact order:
	 * Integer characteristicID, Integer value, String symbol, String relation.
	 
	public void saveRuleInDB(Connection conn, ActionErrors errors, Vector ruleElements) 
	{ 
		try {	
			PreparedStatement stmt = conn.prepareStatement("insert into T_RULE (ELEMENT_ID, NEXT_ELEMENT_ID, RELATION) values (?, ?, ?)");
			int elementId = 0;
			int next_elementId = 0;
			String relation = "";
			
			for (int i=0; i<ruleElements.size(); i+=4) {
				int characId = ((Integer)ruleElements.elementAt(i)).intValue();
				int value = ((Integer)ruleElements.elementAt(i+1)).intValue();
				String symbol = (String)ruleElements.elementAt(i+2);
				//First time, store relationship untill next ruleelement
				if(i==0) {
					elementId = saveRuleElementInDB(conn, errors, characId, value, symbol);
					relation = (String)ruleElements.elementAt(i+3);
					
				//If this is not the first element, store a rule between the two characterisations
				} else {
					next_elementId = saveRuleElementInDB(conn, errors, characId, value, symbol);
					if(next_elementId != -1 && elementId != -1) {
						stmt.setInt(1, elementId);
						stmt.setInt(2, next_elementId);
						stmt.setString(3,""+relation);
						stmt.executeUpdate();
					}
					elementId = next_elementId;
					relation = (String)ruleElements.elementAt(i+3);									
				}
	 		}	
	 	} catch(SQLException sqle) {
		   	errors.add("error.database", new ActionError("error.database"));
	   	}	
	}
	*/
	
	/** 
	 * Checks whether this characterisation conforms to the characterisation rules stored in the database.
	 * 
	 * @param conn A connection with the database
	 * @param errors An ActionError object to store occurring errors in
	 * @param characteristics A Vector of CharacteristicBean objects  PERHAPS THIS SHOULD BE THEIR IDS ONLY?!?
	 * @param values An int[] of scores of the characteristics on a certain task
	 * @param taskid An int representing the id of the task characterised in the database
	 * 
	 * @return A Vector of warnings if any of the characterisations go against the rules, or an empty Vector if they all conform to the rules.
	 
	public Vector doRuleCheck (Connection conn, ActionErrors errors, Vector characteristics, int[] values, int taskid) 
	{
		Vector ruleWarnings = new Vector();
		try {
			for(int i=0; i<characteristics.size(); i++) {
				CharacteristicBean charac = (CharacteristicBean)characteristics.elementAt(i);
				int characID = charac.getCharacteristicID();
				int sco = charac.getScore();
				//Check if there are any rules for this characteristic that apply to this characterisation value in the database
				Vector rule_ids = checkForRulesForCharacteristic(conn, errors, characID, sco);
				if(!rule_ids.isEmpty()) {
					
					PreparedStatement pstmt = conn.prepareStatement("select START_OR_STOP, ELEMENT_ID, NEXT_ELEMENT_ID, RELATION from T_RULE where RULE_ID = ?");
					//get the rule ids
					for (int k=0; k<rule_ids.size(); k++) {
						int ruleid = ((Integer)rule_ids.elementAt(k)).intValue();
						pstmt.setInt(1, ruleid);
						ResultSet rs = pstmt.executeQuery();
						Vector rule = new Vector();
						while(rs.next()) {
							String startstop = rs.getString(1);
							
							
						}
					}
					//Vector rulesLeft = this.getRulesLeftOfThis(conn, errors, rs.getInt(1));
					//Vector rulesRight = this.getRulesRightOfThis(conn, errors, rs.getInt(1));
					//getRulesForCharacteristicFromDB(conn, errors, characID);
					
						
				
					//check rule conformance
				}
			} 
		} catch(SQLException sqle) {
		 	errors.add("error.database", new ActionError("error.database"));
	   	}
	   	return ruleWarnings;	
	}
	*/
	
	/**
	 * Checks if there are any rules stored for this characteristic in the database
	 * @param conn A connection with the database
	 * @param errors
	 * @param characteristicId The id of the characteristic in the database.
	 * @param characScore The score of this characteristic on a certain task
	 * @return The rule ids of the rules that this characteristic is involved in if there exist any, an empty Vector otherwise
	 
	public Vector checkForRulesForCharacteristic(Connection conn, ActionErrors errors, int characteristicId, int characScore)
	{
		Vector ruleIds = new Vector();
		try {
			PreparedStatement stmtEquals = conn.prepareStatement("select ELEMENT_ID from T_RULE_ELEMENT where CHARACTERISTIC_ID = ? and VALUE = ?");
			PreparedStatement stmtNotEquals = conn.prepareStatement("select ELEMENT_ID from T_RULE_ELEMENT where CHARACTERISTIC_ID = ? and VALUE <> ?");
			PreparedStatement stmtLessThan = conn.prepareStatement("select ELEMENT_ID from T_RULE_ELEMENT where CHARACTERISTIC_ID = ? and VALUE > ?");
			PreparedStatement stmtBiggerThan = conn.prepareStatement("select ELEMENT_ID from T_RULE_ELEMENT where CHARACTERISTIC_ID = ? and VALUE < ?");
			PreparedStatement stmtLessOrEqual = conn.prepareStatement("select ELEMENT_ID from T_RULE_ELEMENT where CHARACTERISTIC_ID = ? and VALUE >= ?");
			PreparedStatement stmtBiggerOrEqual = conn.prepareStatement("select ELEMENT_ID from T_RULE_ELEMENT where CHARACTERISTIC_ID = ? and VALUE <= ?");
			ResultSet res;
					
			Statement stmt = conn.createStatement();
			ResultSet rs = stmt.executeQuery("select SYMBOL, VALUE from T_RULE_ELEMENT where CHARACTERISTIC_ID = "+characteristicId);
			
			//Checking if the rule can be applied to the characterisation of this task
			while(rs.next()){
				String symbol = rs.getString(1);
				if(symbol.equals("=")) {
					stmtEquals.setInt(1, characteristicId);
					stmtEquals.setInt(2, characScore);
					res = stmtEquals.executeQuery();
				} else if(symbol.equals("!=")) {
					stmtNotEquals.setInt(1, characteristicId);
					stmtNotEquals.setInt(2, characScore);
					res = stmtNotEquals.executeQuery();
				} else if(symbol.equals("<")) {
					stmtLessThan.setInt(1, characteristicId);
					stmtLessThan.setInt(2, characScore);
					res = stmtLessThan.executeQuery();	
				} else if(symbol.equals(">")) {
					stmtBiggerThan.setInt(1, characteristicId);
					stmtBiggerThan.setInt(2, characScore);	
					res = stmtBiggerThan.executeQuery();
				} else if(symbol.equals("<=")) {
					stmtLessOrEqual.setInt(1, characteristicId);
					stmtLessOrEqual.setInt(2, characScore);
					res = stmtLessOrEqual.executeQuery();
				} else if(symbol.equals(">=")){
					stmtBiggerOrEqual.setInt(1, characteristicId);
					stmtBiggerOrEqual.setInt(2, characScore);
					res = stmtBiggerOrEqual.executeQuery();
				}
				
				//If the rule can be applied
				if(res.next()) {
					
					
				
				}
			}		
			
			
			
			rs = stmt.executeQuery("select distinct T_RULE.RULE_ID from T_RULE, T_RULE_ELEMENT where T_RULE_ELEMENT.CHARACTERISTIC_ID = "+characteristicId+" and (T_RULE_ELEMENT.ELEMENT_ID = T_RULE.ELEMENT_ID or T_RULE_ELEMENT.ELEMENT_ID = T_RULE.NEXT_ELEMENT_ID)");
			while(rs.next())
				ruleIds.addElement(new Integer(rs.getInt(1)));
			
		} catch(SQLException sqle) {
		   	errors.add("error.database", new ActionError("error.database"));
	   	}
	   	return ruleIds;	
	}
	*/
	
	/**
	 * Gets the rule elements to the left of the rule element specified by the element id
	 *
	 * @param conn
	 * @param errors
	 * @param elementID The id of the rule element in the database
	 * @return A Vector of the rule element ids as Integers
	 
	public Vector getRulesLeftOfThis(Connection conn, ActionErrors errors, int elementID)
	{
		Vector leftRules = new Vector();
		try {		
			Statement stmt = conn.createStatement();
			ResultSet rs = stmt.executeQuery("select ELEMENT_ID from T_RULE where NEXT_ELEMENT_ID = "+elementID);
			if(rs.next()) {
				int elementId = rs.getInt(1);
				leftRules = getRulesLeftOfThis(conn, errors, elementId);
				leftRules.addElement(new Integer(elementID));
			}
		} catch(SQLException sqle) {
		   	errors.add("error.database", new ActionError("error.database"));
	   	}
		return leftRules;
	}
	 */
	 
	 /**
	 * Gets the rule elements to the right of the rule element specified by the element id
	 *
	 * @param conn
	 * @param errors
	 * @param elementID The id of the rule element in the database
	 * @return A Vector of the rule elements' ids as Integers
	 
	public Vector getRulesRightOfThis(Connection conn, ActionErrors errors, int elementID)
	{
		Vector rightRules = new Vector();
		try {		
			Statement stmt = conn.createStatement();
			ResultSet rs = stmt.executeQuery("select NEXT_ELEMENT_ID from T_RULE where ELEMENT_ID = "+elementID);
			if(rs.next()) {
				int elementId = rs.getInt(1);
				rightRules = getRulesRightOfThis(conn, errors, elementId);
				rightRules.addElement(new Integer(elementID));
			}
		} catch(SQLException sqle) {
		   	errors.add("error.database", new ActionError("error.database"));
	   	}
		return rightRules;
	}
	*/
	
	/** 
	 * Checks if there are any rules for this characteristic in the database
	 *
	 * @param characteristicId The id of the characteristic to get the applying rules for
	 * @param conn A connection with the database
	 * @param A Vector of either (1) the ids of the characteristics involved in this rule, or (2) an empty Vector if there are no rules for this characteristic
	
	public Vector checkForRulesForCharacteristicInDB(Connection conn, ActionErrors errors, int characteristicId)
	{
		Vector involved = new Vector();
		try {		
			Statement stmt = conn.createStatement();
			ResultSet rs = stmt.executeQuery("select ELEMENT_ID from T_RULE_ELEMENT where CHARACTERISTIC_ID = "+characteristicId);
			
			while(rs.next()){
				int elementId = rs.getInt(1);
										
				Statement st = conn.createStatement();
				ResultSet res = stmt.executeQuery("select T_RULE_ELEMENT.CHARACTERISTIC_ID from T_RULE_ELEMENT, T_RULE where T_RULE_ELEMENT.ELEMENT_ID = T_RULE.ELEMENT_ID and T_RULE.NEXT_ELEMENT_ID = "+elementId);
				while(res.next()) {
					involved.addElement(new Integer(res.getInt(1)));
					Vector involvedCharacs = checkForRulesForCharacteristicInDB(conn, errors, res.getInt(1));
					involved.addAll(involvedCharacs);
				}
				res = stmt.executeQuery("select T_RULE_ELEMENT.CHARACTERISTIC_ID from T_RULE_ELEMENT, T_RULE where T_RULE_ELEMENT.ELEMENT_ID = T_RULE.NEXT_ELEMENT_ID and T_RULE.ELEMENT_ID = "+elementId);
				while(res.next()) {
					involved.addElement(new Integer(res.getInt(1)));
					Vector involvedCharacs = checkForRulesForCharacteristicInDB(conn, errors, res.getInt(1));
					involved.addAll(involvedCharacs);				
				}				
			}		
		
		} catch(SQLException sqle) {
		   	errors.add("error.database", new ActionError("error.database"));
	   	}
	   	return involved;
	}
	 */ 
}
