package com.mowahs.mwcf.framework;

import java.io.Serializable;
import java.util.Vector;
import java.sql.*;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionError;
 
/**
 * This class represents a group in the MWCF framework. A group consists of one or more characteristics.
 * The class handles all logic tied to group creation, storage and deletion.
 * @author Siv Oma Rogdar
 */
public class GroupBean implements Serializable 
{
	private int groupID;
	private String name;
	private String description;
	private Vector groups;
	private Vector characteristics;
	private int characterised = 0;
	//The number of characteristics in this group
	private int noCharacteristics = 0;
		
	/**
	 */
	public GroupBean() 
	{
		
	}
	
	/**
	 * Gets the id of this group 
	 * @return int
	 */
	public int getGroupID() {
		return this.groupID;
	}
	
	/**
	 * Sets the id of this group
	 */
	public void setGroupID(int groupID) {
		this.groupID = groupID;
	}
	
	/**
	 * Gets the name of this group
	 * @return java.lang.String
	 */
	public String getName() {
		return this.name;
	}
	
	/**
	 * Sets the name of this group
	 */
	public void setName(String name) {
		this.name = name;
	}
	
	/**
	 * Gets the description of this group
	 * @return java.lang.String The description of this group
	 */
	 public String getDescription() {
	 	return this.description;
	}
	
	/**
	 * Sets the description of this group
	 * @param java.lang.String The description of this group
	 */
	 public void setDescription(String description) {
	 	this.description = description;
	} 
	
	/**
	 * Sets the Vector of GroupBean objects representing all the groups in this framework
	 * @param java.util.Vector Contains GroupBean objects
	 */
	public void setGroups(Vector groups) {
		this.groups = groups;
	}
	
	/**
	 * Gets the Vector of GroupBean objects representing all the groups in this framework
	 * @return java.util.Vector The GroupBean objects in this framework
	 */
	 public Vector getGroups() {
	 	return this.groups;
	}

	/** 
	 * Gets the indication whether this group has been characterised(1) or not(0).
	 * @return int 1 if has been characterised. 0 if not.
	 */
	public int getCharacterised() {
		return this.characterised;
	}
	
	/**
	 * Sets the int characterised property indicating whether this characteristic has been characterised or not.
	 * @param int If the characteristic has been characterised (1) or not (0).
	 */
	public void setCharacterised(int characterised) {
		this.characterised = characterised;
	}
	
	/**
	 * Sets the number of characteristics in this group
	 * @param no The number of characteristics in this group as an int
	 */
	public void setNoCharacteristics(int no) {
		this.noCharacteristics = no;
	}
	
	/**
	 * Gets the number of characteristics in this group
	 * @return int The number of characteristics
	 */
	public int getNoCharacteristics() {
		return this.noCharacteristics;
	}

	/**
	 * Get the Vector of CharacteristicBean objects in this group
	 * @return java.util.Vector
	 */
	 public Vector getCharacteristics() {
	 	return this.characteristics;
	}
	
	/** 
	 * Sets the Vector of CharacteristicBean objects in this group
	 */
	public void setCharacteristics(Vector characs) {
		this.characteristics = characs;
	}
	
	
	/**
	 * Gets all GroupBean objects from the database and sets their properties. The groups are 
	 * stored in a Vector of GroupBeans, which can be retrieved by the getGroups-method.
	 * The properties that are set are: 
	 * groupID, name, description and characteristics.
	 * This method does not retrieve any task-related properties (such as characterised).
	 * These properties can then be retrieved by getProperty()-methods.
	 * @param conn A connection with the database
	 * @param errors An ActionErrors object to store any occurring errors in
	 */
	public void getAllGroupsFromDB(Connection conn, ActionErrors errors) 
	{
		try {
   			Statement stmt = conn.createStatement();
	       	ResultSet rs = stmt.executeQuery("select * from T_GROUP");
	       	
	       	Vector allgroups = new Vector();
        	while(rs.next()) {
        		GroupBean agroup = new GroupBean();
        		agroup.setGroupID(rs.getInt("GROUP_ID"));
        		agroup.setName(rs.getString("NAME"));
        		agroup.setDescription(rs.getString("DESCRIPTION"));
        		agroup.getCharacteristicsInGroupFromDB(conn, errors);
        		allgroups.add(agroup);
        	} 
        	this.setGroups(allgroups);   		
  
        } catch(SQLException sqle) {
	    	errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("database.could.not.connect"));
   		}	
		
	}
	
	
	/**
	 * Gets all GroupBean objects from the database and sets their properties. 
	 * The properties that are set are: 
	 * groupID, name, description, characteristics and characterised.
	 * These properties can then be retrieved by getProperty()-methods.
	 * @param conn A connection with the database
	 * @param errors An ActionErrors object to store any occurring errors in
	 * @param taskID The id of the task in the database.
	 */
	public void getAllGroupsFromDB(Connection conn, ActionErrors errors, int taskID) 
	{
		try {
   			Statement stmt = conn.createStatement();
	       	ResultSet rs = stmt.executeQuery("select * from T_GROUP");
	       	
	       	Vector allgroups = new Vector();
        	while(rs.next()) {
        		GroupBean agroup = new GroupBean();
        		agroup.setGroupID(rs.getInt("GROUP_ID"));
        		agroup.setName(rs.getString("NAME"));
        		agroup.setDescription(rs.getString("DESCRIPTION"));
        		agroup.getCharacteristicsInGroupFromDB(conn, errors, taskID);
        		allgroups.add(agroup);
        	} 
        	this.setGroups(allgroups);   		
        
        } catch(SQLException sqle) {
	    	errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("database.could.not.connect"));
   		}	
		
	}
	
	
	/**
	 * Removes the group permanently from the database. This method also removes any characteristics in 
	 * this group from the db. That means that all characterisation done on this characteristic are removed, also 
	 * the connection of a characterstic and the scale it uses is removed. 
	 *
	 * @param groupId The int = the id of the group to remove
	 * @param conn A connection with the database
	 * @param errors An ActionErrors object to store any occurring errors in
	 */
	public void removeGroupFromDB(Connection conn, ActionErrors errors, int groupId) 
	{
		try {
   			PreparedStatement stmtSC = conn.prepareStatement("select CHARACTERISTIC_ID from T_CHARACTERISTIC where GROUP_ID = ?");
			stmtSC.setInt(1, groupId);
			ResultSet rs = stmtSC.executeQuery();
			
			//PreparedStatement stmtTC = conn.prepareStatement("delete from T_TASK_CHAR where CHARACTERISTIC_ID = ?");
			//PreparedStatement stmtI = conn.prepareStatement("delete from T_INVOLVEMENT where CHARACTERISTIC_ID = ?");
			//PreparedStatement stmtCV = conn.prepareStatement("delete from T_CHAR_VALUE where CHARACTERISTIC_ID = ?");
			//PreparedStatement stmtC = conn.prepareStatement("delete from T_CHARACTERISTIC where CHARACTERISTIC_ID = ?");
			int i = 0;
			CharacteristicBean charac = new CharacteristicBean();
			while(rs.next()) {
				charac.removeCharacteristicFromDB(conn, errors, rs.getInt(1));
				
				/*
				int charID = rs.getInt("CHARACTERISTIC_ID");
				stmtTC.setInt(1, charID);
				i = stmtTC.executeUpdate();
				
				stmtI.setInt(1, charID);
				i = stmtI.executeUpdate();
				
				stmtCV.setInt(1, charID);
				i = stmtCV.executeUpdate();
			
				stmtC.setInt(1, charID);
				i = stmtC.executeUpdate();
				*/
			}
				
			PreparedStatement stmt = conn.prepareStatement("delete from T_GROUP where GROUP_ID = ?");
			stmt.setInt(1, groupId);
			i = stmt.executeUpdate();
	       	
	       	if(i != 1)
	       		errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("database.not.deleted"));
	       		
        } catch(SQLException sqle) {
	    	errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("database.could.not.connect"));
   		}
	}
	
	/**
	 * Gets the characteristics in this group from the database as CharacteristicBean objects.
	 * These characteristic objects are stored in a Vector on this group and can be retrieved 
	 * by the getCharacteristics() method. The method also gets the scales of each characteristic.
	 * This method does not retrieve any task-related properties (does not check whether
	 * a task has been characterised by the characteristics or not).
	 * @param conn A connection with the database
	 * @param errors An ActionErrors object to store any occurring errors in
	 */
	public void getCharacteristicsInGroupFromDB(Connection conn, ActionErrors errors) {
		
		try {
			PreparedStatement stmt = conn.prepareStatement("select * from T_CHARACTERISTIC where GROUP_ID = ?");
			stmt.setInt(1, this.groupID);
			ResultSet rs = stmt.executeQuery();
									
	       	Vector groupCharacs = new Vector();
	       	int bo = 0;
        	while(rs.next()) {
        		CharacteristicBean charac = new CharacteristicBean();
        		charac.setCharacteristicID(rs.getInt("CHARACTERISTIC_ID"));
        		charac.setName(rs.getString("NAME"));
        		charac.setDescription(rs.getString("DESCRIPTION"));
        		charac.getScalesFromDB(conn, errors);
        		groupCharacs.add(charac);
        	} 
        	this.setCharacteristics(groupCharacs);   
        			
        } catch(SQLException sqle) {
	    	errors.add("error.database", new ActionError("error.database"));
   		}	
		
	}
	
	/**
	 * Gets the characteristics in this group from the database as CharacteristicBean objects.
	 * These characteristic objects are stored in a Vector on this group and can be retrieved 
	 * by the getCharacteristics() method. The method also gets the scales of 
	 * each characteristic and checks whether
	 * a task has been characterised by the characteristic or not.
	 * @param conn A connection with the database
	 * @param errors An ActionErrors object to store any occurring errors in
	 * @param int The task id
	 */
	public void getCharacteristicsInGroupFromDB(Connection conn, ActionErrors errors, int taskID) 
	{
		try {
			PreparedStatement stmt = conn.prepareStatement("select * from T_CHARACTERISTIC where GROUP_ID = ?");
			stmt.setInt(1, this.groupID);
			ResultSet rs = stmt.executeQuery();
									
	       	Vector groupCharacs = new Vector();
	       	int bo = 0;
	       	while(rs.next()) {
        		CharacteristicBean charac = new CharacteristicBean();
        		charac.setCharacteristicID(rs.getInt("CHARACTERISTIC_ID"));
        		charac.setName(rs.getString("NAME"));
        		charac.setDescription(rs.getString("DESCRIPTION"));
        		charac.getScalesFromDB(conn, errors);
        		boolean temp = charac.getCharacterisationFromDB(conn, errors, taskID);
        		if(temp)
        			bo = 1;
  				groupCharacs.add(charac);
        	} 
        	this.setCharacterised(bo);
        	this.setCharacteristics(groupCharacs);
        	int si = groupCharacs.size();
        	this.setNoCharacteristics(si);   
        			
        } catch(SQLException sqle) {
	    	errors.add("error.database", new ActionError("error.database"));
   		}	
		
	}
	
	
	/**
	 * Stores a new group in the database. Returns the newly created unique id of the group in the database.
	 * The GroupBean object must have the name and description properties set before this method is used.
	 * See: setName() and setDescription()
	 * @return int The unique id of the new group in the database
	 * @param conn A connection with the database
	 * @param errors An ActionErrors object to store any occurring errors in
	 */
	public int saveGroupInDB(Connection conn, ActionErrors errors) 
	{
		int groupid = -1;
		try {
   			PreparedStatement stmt = conn.prepareStatement("insert into T_GROUP (NAME, DESCRIPTION) values (?, ?)");
			stmt.setString(1, this.name);
			stmt.setString(2, this.description);
	       	boolean bool = stmt.execute();
	       	
	       	if(!bool){
	       		Statement stmtid = conn.createStatement();
	       		ResultSet rs = stmtid.executeQuery("select LAST_INSERT_ID() as lastid");
	       		if(rs.next())
	       			groupid = rs.getInt("lastid");
			}
			
        } catch(SQLException sqle) {
	    	errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("database.could.not.connect"));
   		}	
		return groupid;
	}
	
	/**
	 * Updates a group's name and description in the database.
	 * @param conn A connection with the database
	 * @param errors An ActionErrors object to store any occurring errors in
	 */
	public void updateGroupInDB(Connection conn, ActionErrors errors) {
		
		try {
   			PreparedStatement stmt = conn.prepareStatement("update T_GROUP set NAME=?, DESCRIPTION=? where GROUP_ID = ?");
			stmt.setString(1, this.name);
			stmt.setString(2, this.description);
	       	stmt.setInt(3, this.groupID);
	       	int i = stmt.executeUpdate();
	       	
	       	if(i != 1){
	       		errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("error.database"));
			}
        } catch(SQLException sqle) {
	    	errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("database.could.not.connect"));
   		}	
   		
	}		  	
	
}
