package com.mowahs.mwcf.framework;

import java.io.Serializable;
import java.sql.*;
import javax.sql.DataSource;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionError;

/**
 * This class represents a role in the MWCF framework. A task has one role as the responsible
 * of the task, and may have dependencies to other tasks. This class handles all logic tied to role
 * creation, storage and deletion.
 * @author Siv Oma Rogdar
 */
public class RoleBean implements Serializable 
{
	//The name of the role
	private String name;
	//The id of the role in the database
	private int roleID;
			
	
	public RoleBean() 
	{	
	}
	
	/**
	 * Get the name of the role
	 * @return java.lang.String
	 */
	public String getName() {
		return this.name;
	}
	
	/**
	 * Set the name of this role
	 * @param name Name of role
	 */
	public void setName(String name) {
		this.name = name;
	}
	
	/**
	 * Set the role id
	 * @param roleID The id of this role in the database
	 */
	public void setRoleID(int roleID) {
		this.roleID = roleID;
	}
	
	/**
	 * Get the role id
	 * @return The role id as an int
	 */
	public int getRoleID() {
		return this.roleID;
	}
		
	/**
	 * Saves this role in the database and sets the roleID property of this role.
	 *
	 * @param scenarioID The id of the scenario that the role belongs to
	 * @param conn A connection with the database
	 * @param errors An ActionErrors object to store any occurring errors in
	 */
	public void saveRoleInDB(Connection conn, ActionErrors errors, int scenarioID) 
	{
		try {
			Statement stmt = conn.createStatement();
			int rows = stmt.executeUpdate("insert into T_ROLE (NAME, SCENARIO_ID) values (\""+this.name+"\", "+scenarioID+")");
	       		       	
	       	if(rows == 1) {
	       		ResultSet rs = stmt.executeQuery("select LAST_INSERT_ID() as lastid");
	        	if(rs.next())
	        	    this.setRoleID(rs.getInt("lastid"));
	        	else
	        		errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("error.database"));
	    	} else
	    		errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("error.database"));
	     
   		} catch(SQLException sqle) {
	    	errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("error.database"));
   		}		
	}
	
	/**
	 * Removes the role completely from the database. 
	 * The method also removes all tasks that this role is responsible for.
	 *
	 * @param conn A connection with the database
	 * @param errors An ActionErrors object to store any occurring errors in
	 * @param role_id The id of the role to be deleted
	 */
	public void removeRoleFromDB(Connection conn, ActionErrors errors, int role_id) 
	{
		try {
   			Statement stmt = conn.createStatement();
			ResultSet rs = stmt.executeQuery("select TASK_ID from T_TASK where ROLE_ID = "+role_id);
			
			TaskBean task = new TaskBean();
			while(rs.next()) {
				task.removeTaskFromDB(conn, errors, rs.getInt(1));
			}
			//Delete role from T_DEPENDENCIES
			stmt.executeUpdate("delete from T_DEPENDENCIES where ROLE_ID = "+role_id);
			
			//Delete role from T_ROLE
			int rows = stmt.executeUpdate("delete from T_ROLE where ROLE_ID = "+role_id);
	       	if(rows != 1)
	       		errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("error.database"));
	       		       	
   		} catch(SQLException sqle) {
	    	errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("error.database"));
   		}		
	}	
	
	/**
	 * Saves a new role in the database and returns the newly created id of this role
	 *
	 * @param conn A connection with the database
	 * @param errors An ActionErrors object to store any occurring errors in
	 * @param scenarioID The id of the scenaro this role belongs to
	 */
	public int saveRoleGetRoleID(Connection conn, ActionErrors errors, int scenarioID) 
	{
		this.setRoleID(0);
		try {
   			Statement stmt = conn.createStatement();
   			int rows = stmt.executeUpdate("insert into T_ROLE (NAME, SCENARIO_ID) values (\""+this.name+"\", "+scenarioID+")");
	       	
	       	if(rows == 1) {	       	
	       		ResultSet rs = stmt.executeQuery("select LAST_INSERT_ID() as lastid");
	       		if(rs.next())
        	    	this.setRoleID(rs.getInt("lastid"));
        	    else
        			errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("error.database"));

        	} else
        		errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("error.database"));
        		
        } catch(SQLException sqle) {
	    	errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("error.database"));
   		}		
	return this.getRoleID();
	}
	
	
	/**
	 * Gets the role from the database and sets all the properties of the role.
	 * The properties that are stored are: int roleID and String roleDescription. The properties can be
	 * retrieved by getProperty() methods.
	 *
	 * @param conn A connection with the database
	 * @param errors An ActionErrors object to store any occurring errors in
	 * @param roleID The int representing the id of the role in the database.
	 */
	public void getRoleFromDB(Connection conn, ActionErrors errors, int roleID) {
	
		try {
   			PreparedStatement stmt = conn.prepareStatement("select NAME from T_ROLE where ROLE_ID = ?");
	       	stmt.setInt(1,roleID);
	       	ResultSet rs = stmt.executeQuery();
	       	
	       	if(rs.next()) {
        		this.setRoleID(roleID);
        		this.setName(rs.getString(1));
        	} else {
        		errors.add("error.database", new ActionError("error.database"));
    		}
    		
        } catch(SQLException sqle) {
	    	errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("sql.exception"));
   		}		
		
		
	}
	
}
