package com.mowahs.mwcf.framework;

import java.util.Vector;
import java.io.Serializable;
import java.sql.*;
import javax.sql.DataSource;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionError;
/**
 * This class represents a rule in the MWCF framework. A rule consists of two or more rule fragments.
 * The class handles logic tied to creation, deletion and storage of rules.
 */
public class RuleBean implements Serializable {
	
	private int ruleFragmentID;
	private int ruleID;
	private int characteristicID;
	// The name of the characteristic
	private String characteristic;
	private int value;
	
	private String name;
	private String warning;
	// The relation to next fragment
	private String relation;
	// The operator connecting the value and the characteristic
	private String operator;
	//A vector containing the rule names
	private Vector rulelist;
	// A vector containing the rule fragments
	private Vector rules;
	
	public RuleBean() {
	}
	/**
	 * Gets the id of this rule fragment
	 * @return The id of this rule fragment
	 */
	public int getRuleFragmentID() {
		return ruleFragmentID;
	}
	
	public int getRuleID() {
		return ruleID;
	}
	
	public void setRuleID(int RuleID) {
		this.ruleID = ruleID;
	}
	/**
	 * Gets the characteristic id in this rule fragment
	 * @return The characteristic id
	 */
	public int getCharacteristicID() {
		return characteristicID;
	}
	/**
	 * Sets the characteristic id in this rule fragment
	 * @param int The id of the characteristic
	 */
	public void setCharacteristicID(int characID) {
		this.characteristicID = characID;
	}
	/**
	 * Gets the characteristic name in this rule fragment
	 * @return The characteristic name
	 */
	public String getCharacteristic() {
		return characteristic;
	}
	/**
	 * Sets the characteristic name in this rule fragment
	 * @param characteristic The name of the characteristic
	 */
	public void setCharacteristic(String characteristic) {
		this.characteristic = characteristic;
	}
	/**
	 * Gets the value in this rule fragment
	 * @return The value in this rule fragment
	 */
	public int getValue() {
		return value;
	}
	/**
	 * Sets the value in this rule fragment
	 * @param int The value connected with the characteristic
	 */
	public void setValue(int value) {
		this.value = value;
	}
	/**
	 * Gets the name of this rule
	 * @return The name of this rule
	 */
	public String getName() {
		return name;
	}
	/**
	 * Sets the name of this rule
	 * @param name The name of the rule
	 */
	public void setName(String name) {
		this.name = name;
	}
	/**
	 * Gets this fragment's relation to the next fragment
	 * @return This fragments relation
	 */
	public String getRelation() {
		return relation;
	}
	/**
	 * Sets this fragment's relation to the next fragment
	 * @param relation The relation to the next fragment
	 */
	public void setRelation(String relation) {
		this.relation = relation;
	}
	/**
	 * Gets the warning in this rule
	 * @return The warning of this rule
	 */
	public String getWarning() {
		return warning;
	}
	/**
	 * Sets the warning in this rule
	 * @param warning The warning in this rule
	 */
	public void setWarning(String warning) {
		this.warning = warning;
	}
	/**
	 * Gets this fragment's operator
	 * @return This fragments operator
	 */
	public String getOperator() {
		return operator;
	}
	/**
	 * Sets this fragment's operator
	 * @param operator The operator in this fragment
	 */
	public void setOperator(String operator) {
		this.operator = operator;
	}
	/**
	 * Sets alle the names of the rules
	 * @param rulelist Sets the names of the rules
	 */
	public void setRuleList(Vector rulelist) {
		this.rulelist = rulelist;
	}
	/**
	 * Gets alle the names of the rules
	 * @return The names of the rules in a Vector
	 */
	public Vector getRuleList() {
		return rulelist;
	}
	/**
	 * Sets all rules
	 * @param rules A Vector containing the rules
	 */
	public void setRules(Vector rules) {
		this.rules = rules;
	}
	/**
	 * Gets all rules
	 * @return All rules in a Vector
	 */
	public Vector getRules() {
		return rules;
	}
	
	
	/**
	 *
	 **/
	 
	public void getRuleFromDB(Connection conn, ActionErrors errors, String name) {
		
		try{
			Statement stmt = conn.createStatement();
			ResultSet rs = stmt.executeQuery("select * from T_RULE where NAME = \""+name+"\"order by RULEFRAGMENT_ID");
			Vector tempRules = new Vector();
			//Vector tempNames = new Vector();
			String tempName = ""; 
			
			while(rs.next()) {
				RuleBean rule = new RuleBean();
				rule.setCharacteristicID(rs.getInt("CHARACTERISTIC_ID"));
				rule.setCharacteristic(rs.getString("CHARACTERISTIC"));
				rule.setValue(rs.getInt("VALUE"));
				rule.setRelation(rs.getString("RELATION"));
				rule.setOperator(rs.getString("OPERATOR"));
				rule.setName(rs.getString("NAME"));
				rule.setWarning(rs.getString("WARNING"));
			//	if(!tempName.equals(rs.getString("NAME"))){
			//		tempName = rs.getString("NAME");
			//		tempNames.add(rule);	 
			//	}
				
				tempRules.add(rule);
			}
			this.setRules(tempRules); 
			//this.setRuleList(tempNames);  
		}
		catch(SQLException sqle) {
	    	errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("error.database"));
   		}
		
	}
	
	/**
	 * Gets all RuleBean objects from the database and sets their properties.
	 * The rule fragments are stored in a vector of RuleBeans
	 * @param conn A connection with the database
	 * @param errors An ActionErrors object to store any occurring errors in
	 */
	 
	public void getAllRulesFromDB(Connection conn, ActionErrors errors) {
		
		try{
			Statement stmt = conn.createStatement();
			ResultSet rs = stmt.executeQuery("select * from T_RULE order by RULEFRAGMENT_ID");
			Vector tempRules = new Vector();
			Vector tempNames = new Vector();
			String tempName = ""; 
			
			while(rs.next()) {
				RuleBean rule = new RuleBean();
				rule.setCharacteristicID(rs.getInt("CHARACTERISTIC_ID"));
				rule.setCharacteristic(rs.getString("CHARACTERISTIC"));
				rule.setValue(rs.getInt("VALUE"));
				rule.setRelation(rs.getString("RELATION"));
				rule.setOperator(rs.getString("OPERATOR"));
				rule.setName(rs.getString("NAME"));
				rule.setWarning(rs.getString("WARNING"));
				if(!tempName.equals(rs.getString("NAME"))){
					tempName = rs.getString("NAME");
					tempNames.add(rule);	 
				}
				
				tempRules.add(rule);
			}
			this.setRules(tempRules); 
			this.setRuleList(tempNames);  
		}
		catch(SQLException sqle) {
	    	errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("error.database"));
   		}
	}
	/**
	 * Deletes all rulefragments with a specific anme from the database, resulting in the deletion of one rule.
	 * @param conn A connection with the database
	 * @param errors An ActionErrors object to store any occurring errors in
	 * @param name A String containing the name of the rule to be deleted
	 */
	public void removeRuleFromDB(Connection conn, ActionErrors errors, String name) {
		
		try {
   			Statement stmt = conn.createStatement();
   			stmt.executeUpdate("delete from T_RULE where NAME = \""+name+"\"");			
        } 
        catch(SQLException sqle) {
	    	errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("error.database"));
   		}	
	}
	/**
	 * Saves a rule fragment to the database
	 * @param conn A connection with the database
	 * @param errors An ActionErrors object to store any occurring errors in
	 */
	public void saveRuleInDB(Connection conn, ActionErrors errors) {
		
		try {
   			Statement stmt = conn.createStatement();
   			ResultSet rs = stmt.executeQuery("select CHARACTERISTIC_ID from T_CHARACTERISTIC where NAME=\""+this.characteristic+"\"");
   			rs.next();
   			this.setCharacteristicID(rs.getInt("CHARACTERISTIC_ID")); 
   			
   			stmt.executeUpdate("insert into T_RULE (RULE_ID, CHARACTERISTIC_ID, CHARACTERISTIC, VALUE, NAME, RELATION, WARNING, OPERATOR) values ("+this.ruleID+","+this.characteristicID+",\""+this.characteristic+"\","+this.value+",\""+this.name+"\",\""+this.relation+"\",\""+this.warning+"\",\""+this.operator+"\")");
   		}
		catch(SQLException sqle) {
	    	errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("error.database"));
   		}		
   	}
}