package com.mowahs.mwcf.framework;

import java.io.Serializable;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;
import java.math.BigDecimal;
import java.sql.*;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionError;
 
/**
 * This class represents a task in the MWCF framework and is responsible for all details of a task
 * in a scenario. A mobile scenario consists of one or more tasks. A task may have pretask(s) and
 * posttask(s), and if the task is composite, it also contains subtask(s). This class handles all logic
 * tied to task creation, storage and deletion.
 * @author Siv Oma Rogdar
 */
public class TaskBean implements Serializable 
{
	private String name;
	private String description;
	private int taskID;
	private int responsible;
	private String responsibleName;
	private String preLocation;
	private String preResources;
	//A Vector of Integers representing the IDs of the precondition tasks
	private Vector preTasks;
	private String postLocation;
	private String postResources;
	//A Vector of Integers representing the IDs of the postcondition tasks
	private Vector postTasks;
	private String composite;
	//The vector of subtask ids (as Integers)
	private Vector subTasks;
	private Vector roleDependencies;
	private int weight = 0;
	//Contains the characteristic ids as keys (of type Integer) and the values of this task as values (of type Integer).
	private Hashtable characterisation;
	//The average of all characterisation on this task
	private float taskAverage;
	
	/**
	 *
	 */
	public TaskBean() 
	{
		
	}
	
	/**
	 * @param name
	 *
	 */
	public void setName(String name) 
	{
		this.name = name;
	}
	
	/**
	 * @return java.lang.String
	 *
	 */
	public String getName() 
	{
		return this.name;
	}
	
	/**
	 * @param description
	 *
	 */
	public void setDescription(String description) 
	{
		this.description = description;
	}
	
	/**
	 * @return java.lang.String
	 *
	 */
	public String getDescription() 
	{
		return this.description;
	}
	
	/**
	 * @param roleID
	 *
	 */
	public void setResponsible(int roleID) 
	{
		this.responsible = roleID;
	}
	
	/**
	 * @return int
	 *
	 */
	public int getResponsible() {
		return this.responsible;
	}
	
	public void setTaskID(int taskID) {
		this.taskID = taskID;
	}
	
	public int getTaskID() {
		return this.taskID;
	}
	
	public void setResponsibleName(String responsibleName) {
		this.responsibleName = responsibleName;
	}
	
	public String getResponsibleName() {
		return this.responsibleName;
	}
	
	public void setSubTasks(Vector subs) {
		this.subTasks = subs;
	}
	
	public Vector getSubTasks() {
		return this.subTasks;
	}
	
	/**
	 * Sets the characterisation property of this task.
	 * The characterisation Hashtable contains the characteristic ids as keys (type Integer) and 
	 * the scores of this task as values (type Integer)
	 * @param characs The Hashtable of characteristic ids and task scores
	 */
	public void setCharacterisation(Hashtable characs) {
		this.characterisation = characs;
	}
	
	/**
	 * Gets the characterisation property of this task.
	 * The characterisation Hashtable contains the characteristic ids as keys (type Integer) and 
	 * the scores of this task as values (type Integer)
	 * @return The Hashtable of characteristic ids of type Integers and task scores of type Integers
	 */
	public Hashtable getCharacterisation() {
		return this.characterisation;
	}
	
	/**
	 * @param preLocation
	 *
	 */
	public void setPreLocation(String preLocation) 
	{
		this.preLocation = preLocation;		
	}
	
	/**
	 * @return java.lang.String
	 */
	public String getPreLocation() 
	{
		return this.preLocation;
	}
	
	/**
	 * @param preResources
	 */
	public void setPreResources(String preResources) 
	{
		this.preResources = preResources;
	}
	
	/**
	 * @return java.lang.String
	 */
	public String getPreResources() 
	{
		return this.preResources;
	}
	
	/**
	 * @param tasks
	 */
	public void setPreTasks(Vector preTasks) 
	{
		this.preTasks = preTasks;
	}
	
	/**
	 * @return java.util.Vector
	 */
	public Vector getPreTasks() 
	{
		return this.preTasks;
	}
	
	/**
	 * @param postLocation
	 */
	public void setPostLocation(String postLocation) 
	{
		this.postLocation = postLocation;
	}
	
	/**
	 * @return java.lang.String
	 */
	public String getPostLocation() 
	{
		return this.postLocation;
	}
	
	/**
	 * @param postResources
	 */
	public void setPostResources(String postResources) 
	{
		this.postResources = postResources;
	}
	
	/**
	 * @return java.lang.String
	 */
	public String getPostResources() 
	{
		return this.postResources;
	}
	
	/**
	 * @param tasks
	 */
	public void setPostTasks(Vector tasks) 
	{
		this.postTasks = tasks;
	}
	
	/**
	 * @return java.util.Vector
	 */
	public Vector getPostTasks() 
	{
		return this.postTasks;
	}
	
	public void setRoleDependencies(Vector roleIDs) {
		this.roleDependencies = roleIDs;
	}
	
	public Vector getRoleDependencies() {
		return this.roleDependencies;
	}
	
	public void setComposite(String composite) {
		this.composite = composite;
	}
	
	public String getComposite() {
		return this.composite;
	}
	
	public void setWeight(int weight) {
		this.weight = weight;
	}
	
	public int getWeight() {
		return this.weight;
	}
	
	/** 
	 * Set the value of the average of all characterisation done on this task
	 * @param ave The float value of the characterisation average
	 */
	public void setTaskAverage(float ave) {
		this.taskAverage = ave;
	}
	
	/**
	 * Get the average of all characterisation done on this task
	 * @return A float value with 2 decimals, representing the average of all characterisation done on this task
	 */
	public float getTaskAverage() {
		return this.taskAverage;
	}
	
	/** 
	 * Calculates the average of the characterisations of this task and sets this value in the property taskAverage (can be retrieved by method getTaskAverage())
	 * This method first gets and sets the characterisations of this task by method getCharacterisationsOfTaskFromDB().
	 * 
	 * @param conn A connection with the database
	 * @param errors An ActionError object to store any occurring errors in
	 */
	public void calculateTaskAverage(Connection conn, ActionErrors errors) {
		
		this.getCharacterisationsOfTaskFromDB(conn, errors);
		Hashtable characScores = this.getCharacterisation();
		if(characScores != null && !characScores.isEmpty()) {
			Enumeration e = characScores.elements();
			int i=0;
			float tempSum = 0;
			while(e.hasMoreElements()) {
				i++;
				tempSum = tempSum + ((Integer)e.nextElement()).intValue()*1000;		
			}
			if(tempSum > 0 && i > 0)
				tempSum = tempSum/i;
			
			BigDecimal bgtemp = new BigDecimal(tempSum);
   			bgtemp = bgtemp.divide(new BigDecimal(1000), 2, BigDecimal.ROUND_HALF_UP);
			this.setTaskAverage(bgtemp.floatValue());
		}	
	}
	
	/**
	 * Gets the characterisation on all characteristics (if any) of this task.
	 * Stores the characteristic ids as keys (type Integer) and the characterisation values as values (type Integer) in a Hashtable.
	 * This may be retrived by the getCharacterisation-method.
	 * @param conn A connection with the database
	 * @param errors An ActionErrors object to store any occurring errors in
	 * @param taskid The id of the task whose characterisation values are wanted
	 */
	public void getCharacterisationsOfTaskFromDB(Connection conn, ActionErrors errors) {
		try {
			Statement stmt = conn.createStatement();
			ResultSet rs = stmt.executeQuery("select CHARACTERISTIC_ID, VALUE from T_TASK_CHAR where TASK_ID ="+this.getTaskID());
			Hashtable temp = new Hashtable();
			while(rs.next()) {
				temp.put(new Integer(rs.getInt(1)), new Integer(rs.getInt(2)));
			}
			this.setCharacterisation(temp);
		} catch(SQLException sqle) {
			errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("error.database"));
   		}	
		
	}
	/**
	 * Method gets the name of the role specifiec by the roleid from the database
	 *
	 * @param conn A connection with the database
	 * @param errors An ActionErrors object to store any occurring errors in
	 * @param respID The id of the role
	 */
	public void findResponsibleNameInDB(Connection conn, ActionErrors errors, int respID) {
		try {
			Statement stmt = conn.createStatement();
			ResultSet rs = stmt.executeQuery("select NAME from T_ROLE where ROLE_ID = "+respID);
	       		       	
	       	if(rs.next())
        		this.setResponsibleName(rs.getString(1));
        	else
        		errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("error.database"));
    	     
   		} catch(SQLException sqle) {
	    	errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("error.database"));
   		}	
   	}
	
	
	
	/**
	 * Saves the weight score of this task in the database.
	 * This method first deletes any existing weight of this task (in case this is an update) and 
	 * then proceeds to insert the new weight value on this task.
	 *
	 * @param conn A connection with the database
	 * @param errors An ActionErrors object to store any occurring errors in
	 */
	public void saveWeightInDB(Connection conn, ActionErrors errors) {
		
		try {
   			Statement stmt = conn.createStatement();
			int result = stmt.executeUpdate("update T_TASK set WEIGHT = "+this.weight+" where TASK_ID = "+this.taskID);
			if(result != 1)
				errors.add("error.database", new ActionError("error.database"));
										
		} catch(SQLException sqle) {
	    	errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("database.could.not.connect"));
   		}	
	}
	
	/**
	 * Saves the task in the database. All of the tasks properties that should be saved must first have been set.
	 * This method also saves all of the dependencies of a task (pretasks, posttasks, subtasks).
	 * 
	 * @param conn A connection with the database
	 * @param errors An ActionErrors object to store any occurring errors in
	 * @return The id of the newly created task element in the database
	 */
	public int saveTaskInDB(Connection conn, ActionErrors errors) 
	{
		this.setTaskID(-1);
		
		try {
			PreparedStatement stmt = conn.prepareStatement("insert into T_TASK (NAME, DESCRIPTION, PRE_LOCATION, PRE_RESOURCES, POST_LOCATION, POST_RESOURCES, ROLE_ID, COMPOSITE) values (?, ?, ?, ?, ?, ?, ?, ?)");
	       	stmt.setString(1,""+this.name);
	       	stmt.setString(2,""+this.description);
	       	stmt.setString(3,""+this.preLocation);
	       	stmt.setString(4,""+this.preResources);
	       	stmt.setString(5,""+this.postLocation);
	       	stmt.setString(6,""+this.postResources);
	       	stmt.setInt(7,this.responsible);
	       	stmt.setString(8,""+this.composite);
	       	stmt.execute();
	       	
	       	Statement stm = conn.createStatement();
	       	ResultSet rs = stm.executeQuery("select LAST_INSERT_ID() as lastid");
        	if(rs.next()) {
        		this.setTaskID(rs.getInt("lastid"));
        	           	    
        	    //Store any role dependencies
        	    if(this.roleDependencies != null && this.roleDependencies.size() > 0) {
        	    	PreparedStatement stmtRoleDeps = conn.prepareStatement("insert into T_DEPENDENCIES (ROLE_ID, TASK_ID) values (?, "+taskID+")");
        	    	for(int i=0; i<this.roleDependencies.size(); i++) {
	        	    	stmtRoleDeps.setInt(1,((Integer)this.roleDependencies.elementAt(i)).intValue());
        	    		stmtRoleDeps.execute();        	    		
        	    	}
				} 
										       	    
				//COMMON to PRE and POST
				PreparedStatement stmtTaskConds = conn.prepareStatement("insert into T_TASK_DEPENDENCY (TASK_ID, DEPENDENT_TASK_ID, CONDITION_TYPE) values (?, ?, ?)");
        	    //STORE TASK PRECONDITIONS
        	    if(this.preTasks != null && !this.preTasks.isEmpty()) {
        	    	for(int a=0; a<this.preTasks.size(); a++) {
        	    		stmtTaskConds.setInt(1,this.taskID);
        	    		stmtTaskConds.setInt(2,((Integer)this.preTasks.elementAt(a)).intValue());
        	    		stmtTaskConds.setString(3,"PRE");
        	    		stmtTaskConds.execute();
        	    	}
        	    } 
        	    
        	    //STORE TASK POSTCONDITIONS
        	    if(this.postTasks != null && !this.postTasks.isEmpty()) {
        	    	for(int a=0; a<this.postTasks.size(); a++) {
        	    		stmtTaskConds.setInt(1,this.taskID);
        	    		stmtTaskConds.setInt(2,((Integer)this.postTasks.elementAt(a)).intValue());
        	    		stmtTaskConds.setString(3,"POST");
        	    		stmtTaskConds.execute();
        	    	}
 				}       
        	    
        	    //STORE COMPOSITE DEPENDENCIES
        	    if((this.composite.equals("Yes")) && (this.subTasks != null) && (!this.subTasks.isEmpty()) && (this.subTasks.size() > 0)) {
        	    	for(int a=0; a<this.subTasks.size(); a++) {
        	    		stmtTaskConds.setInt(1,this.taskID);
        	    		stmtTaskConds.setInt(2,((Integer)this.subTasks.elementAt(a)).intValue());
        	    		stmtTaskConds.setString(3,"COMPOSITE");
        	    		stmtTaskConds.execute();	
        	    	}	
        	    } 
        	    
        	} else {
        		errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("error.database", name));
    		}
		     
   		} catch(SQLException sqle) {
	    	errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("database.could.not.connect"));
   		}	
   		return this.taskID;   	
	}
	
	/**
	 * Update the task in the database. All of the tasks properties must first be set.
	 * Also updates all of the dependencies of a task (pretasks, posttasks, subtasks).
	 *
	 * @param conn A connection with the database
	 * @param errors An ActionErrors object to store any occurring errors in
	 * @param taskId The id of the task to be updated.
	 */
	public void updateTaskInDB(Connection conn, ActionErrors errors, int taskId) 
	{
		try {
			PreparedStatement stmt = conn.prepareStatement("update T_TASK set NAME = ?, DESCRIPTION = ?, PRE_LOCATION = ?, PRE_RESOURCES = ?, POST_LOCATION = ?, POST_RESOURCES = ?, ROLE_ID = ?, COMPOSITE = ? where TASK_ID = ?");
	       	stmt.setString(1,""+this.name);
	       	stmt.setString(2,""+this.description);
	       	stmt.setString(3,""+this.preLocation);
	       	stmt.setString(4,""+this.preResources);
	       	stmt.setString(5,""+this.postLocation);
	       	stmt.setString(6,""+this.postResources);
	       	stmt.setInt(7,this.responsible);
	       	stmt.setString(8,""+this.composite);
	       	stmt.setInt(9,taskId);
	       	int rows = stmt.executeUpdate();
	     	if(rows == 1) {
				Statement state = conn.createStatement();
	     		
	     		//Delete any existing role dependencies
	     		state.executeUpdate("delete from T_DEPENDENCIES where TASK_ID = "+taskID);
	     			     			     			       	   	
	       	   	//Store any role dependencies
        	    if(this.roleDependencies != null && this.roleDependencies.size() > 0) {
        	    	PreparedStatement stmtRoleDeps = conn.prepareStatement("insert into T_DEPENDENCIES (ROLE_ID, TASK_ID) values (?, "+taskID+")");
        	    	for(int i=0; i<this.roleDependencies.size(); i++) {
	        	    	stmtRoleDeps.setInt(1,((Integer)this.roleDependencies.elementAt(i)).intValue());
        	    		stmtRoleDeps.execute();        	    		
        	    	}
				}
					
					       	    
				//COMMON to PRE and POST
				PreparedStatement stmtTaskConds = conn.prepareStatement("insert into T_TASK_DEPENDENCY (TASK_ID, DEPENDENT_TASK_ID, CONDITION_TYPE) values (?, ?, ?)");
        	    
        	    //STORE TASK PRECONDITIONS
        	    if(this.preTasks != null && !this.preTasks.isEmpty()) {
        	    	
        	    	//Delete any existing dependencies where this task has pretasks
        	    	state.executeUpdate("delete from T_TASK_DEPENDENCY where TASK_ID = "+taskID+" and CONDITION_TYPE = \"PRE\"");
        	    	
        	    	for(int a=0; a<this.preTasks.size(); a++) {
        	    		int pretask = ((Integer)this.preTasks.elementAt(a)).intValue();
        	    		
        	    		//Delete any instances where this task is the pretask of one of its pretasks ( PRE(pretask, thistask) ).
        	    		state.executeUpdate("delete from T_TASK_DEPENDENCY where TASK_ID = "+pretask+" and DEPENDENT_TASK_ID = "+taskID+" and CONDITION_TYPE = \"PRE\"");
        	    		
        	    		//Delete any instances where any of this tasks pretasks is its posttask  POST(thistask, pretask)
        	    		state.executeUpdate("delete from T_TASK_DEPENDENCY where TASK_ID = "+taskID+" and DEPENDENT_TASK_ID = "+pretask+" and CONDITION_TYPE = \"POST\"");
        	    		
        	    		//Delete any instances where this task is composed of any of its pretasks or any of its pretasks is composed of this task
        	    		state.executeUpdate("delete from T_TASK_DEPENDENCY where TASK_ID = "+taskID+" and DEPENDENT_TASK_ID = "+pretask+" and CONDITION_TYPE = \"COMPOSITE\"");
        	    		state.executeUpdate("delete from T_TASK_DEPENDENCY where TASK_ID = "+pretask+" and DEPENDENT_TASK_ID = "+taskID+" and CONDITION_TYPE = \"COMPOSITE\"");
        	    		
        	    		stmtTaskConds.setInt(1,this.taskID);
        	    		stmtTaskConds.setInt(2,pretask);
        	    		stmtTaskConds.setString(3,"PRE");
        	    		stmtTaskConds.execute();
        	    	}
        	    } else {
        	    	
        	    	//Delete any existing task dependencies where this task is the posttask or has pretasks
	     			state.executeUpdate("delete from T_TASK_DEPENDENCY where DEPENDENT_TASK_ID = "+taskID+" and CONDITION_TYPE = \"POST\"");	
	     			state.executeUpdate("delete from T_TASK_DEPENDENCY where TASK_ID = "+taskID+" and CONDITION_TYPE = \"PRE\"");	
        	    }
        	    
        	    
        	    //STORE TASK POSTCONDITIONS
        	    if(this.postTasks != null && !this.postTasks.isEmpty()) {
					
					//Delete any existing dependencies where this task has posttasks
        	    	state.executeUpdate("delete from T_TASK_DEPENDENCY where TASK_ID = "+taskID+" and CONDITION_TYPE = \"POST\"");        	    	
        	    	
        	    	for(int a=0; a<this.postTasks.size(); a++) {
						int posttask = ((Integer)this.postTasks.elementAt(a)).intValue();
						
						//Delete any instances where this task is the posttask of any of its posttasks: POST(posttask, thistask)
        	    		state.executeUpdate("delete from T_TASK_DEPENDENCY where TASK_ID = "+posttask+" and DEPENDENT_TASK_ID = "+taskID+" and CONDITION_TYPE = \"POST\"");
        	    		
        	    		//Delete any instances where any of this tasks posttasks is its pretask: PRE(thistask, posttask)
        	    		state.executeUpdate("delete from T_TASK_DEPENDENCY where TASK_ID = "+taskID+" and DEPENDENT_TASK_ID = "+posttask+" and CONDITION_TYPE = \"PRE\"");
        	    		
        	    		//Delete any instances where this task is composed of any of its posttasks or any of its posttasks is composed of this task
        	    		state.executeUpdate("delete from T_TASK_DEPENDENCY where TASK_ID = "+taskID+" and DEPENDENT_TASK_ID = "+posttask+" and CONDITION_TYPE = \"COMPOSITE\"");
        	    		state.executeUpdate("delete from T_TASK_DEPENDENCY where TASK_ID = "+posttask+" and DEPENDENT_TASK_ID = "+taskID+" and CONDITION_TYPE = \"COMPOSITE\"");
        	    		
        	    		stmtTaskConds.setInt(1,this.taskID);
        	    		stmtTaskConds.setInt(2,posttask);
        	    		stmtTaskConds.setString(3,"POST");
        	    		stmtTaskConds.execute();
        	    	}
        	    } else {
        	    	
        	    	//Delete any existing task dependencies where this task is the pretask or has posttasks
        	    	state.executeUpdate("delete from T_TASK_DEPENDENCY where DEPENDENT_TASK_ID = "+taskID+" and CONDITION_TYPE = \"PRE\"");	
	     			state.executeUpdate("delete from T_TASK_DEPENDENCY where TASK_ID = "+taskID+" and CONDITION_TYPE = \"POST\"");
        	    }
        	    
        	    
        	    //STORE COMPOSITE DEPENDENCIES
        	    if((this.composite.equals("Yes")) && (this.subTasks != null) && (!this.subTasks.isEmpty()) && (this.subTasks.size() > 0)) {
        	    	//Delete any existing task dependencies where this task has subtasks
        	    	state.executeUpdate("delete from T_TASK_DEPENDENCY where TASK_ID = "+taskID+" and CONDITION_TYPE = \"COMPOSITE\"");	
        	    	        	    	
        	    	for(int a=0; a<this.subTasks.size(); a++) {
						int subtask = ((Integer)this.subTasks.elementAt(a)).intValue();
						
						//Delete any task dependencies where any of this task's subtasks is composed of this task
						state.executeUpdate("delete from T_TASK_DEPENDENCY where TASK_ID = "+subtask+" and DEPENDENT_TASK_ID = "+taskID+" and CONDITION_TYPE = \"COMPOSITE\"");	
						
						//Delete any task dependencies where the subtask is involved in a pre or post relationship with this task
						state.executeUpdate("delete from T_TASK_DEPENDENCY where TASK_ID = "+taskID+" and DEPENDENT_TASK_ID = "+subtask+" and CONDITION_TYPE = \"PRE\"");
						state.executeUpdate("delete from T_TASK_DEPENDENCY where TASK_ID = "+subtask+" and DEPENDENT_TASK_ID = "+taskID+" and CONDITION_TYPE = \"PRE\"");
						state.executeUpdate("delete from T_TASK_DEPENDENCY where TASK_ID = "+taskID+" and DEPENDENT_TASK_ID = "+subtask+" and CONDITION_TYPE = \"POST\"");
						state.executeUpdate("delete from T_TASK_DEPENDENCY where TASK_ID = "+subtask+" and DEPENDENT_TASK_ID = "+taskID+" and CONDITION_TYPE = \"POST\"");
												        	    		
        	    		stmtTaskConds.setInt(1,this.taskID);
        	    		stmtTaskConds.setInt(2,subtask);
        	    		stmtTaskConds.setString(3,"COMPOSITE");
        	    		stmtTaskConds.execute();	
        	    	}	
        	    } else {
        	    	
        	       	//Delete any existing task dependencies where this task has subtasks
        	    	state.executeUpdate("delete from T_TASK_DEPENDENCY where TASK_ID = "+taskID+" and CONDITION_TYPE = \"COMPOSITE\"");	
        	    }
        	} else {
        		errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("error.database"));
    		}
		     
   		} catch(SQLException sqle) {
	    	errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("sql.exception"));
   		}	
   	}
	
	
	/**
	 * Gets the task with a specific task id from the database and sets all the properties of the task.
	 * The properties that are stored are: 
	 * taskID, description, responsible (the role ID of the responsible), preLocation, preResources, preTasks, 
	 * postLocation, postResources, postTasks, roleDependencies, composite ("Yes" or "No"), subTasks and weight. 
	 * 
	 * @param conn A connection with the database
	 * @param errors An ActionErrors object to store any occurring errors in
	 * @param taskID An int representing the ID of this task
	 */
	 public void getTaskFromDB(Connection conn, ActionErrors errors, int taskID) {
	 
	 	try {
   			PreparedStatement stmt = conn.prepareStatement("select NAME, DESCRIPTION, PRE_LOCATION, PRE_RESOURCES, POST_LOCATION, POST_RESOURCES, ROLE_ID, COMPOSITE, WEIGHT from T_TASK where TASK_ID = ?");
			stmt.setInt(1,taskID);
			ResultSet rs = stmt.executeQuery();
				
			while(rs.next()) {
				this.setTaskID(taskID);
				this.setName(rs.getString(1));
				this.setDescription(rs.getString(2));
				this.setPreLocation(rs.getString(3));
				this.setPreResources(rs.getString(4));
				this.setPostLocation(rs.getString(5));
				this.setPostResources(rs.getString(6));
				this.setResponsible(rs.getInt(7));
				this.setComposite(rs.getString(8));
				this.setWeight(rs.getInt(9));
			}
			
			if(this.taskID > -1) {
				Statement state = conn.createStatement();
				
				//Get all pretasks
				rs = state.executeQuery("select DEPENDENT_TASK_ID from T_TASK_DEPENDENCY where TASK_ID = "+this.taskID+" and CONDITION_TYPE = \"PRE\"");
				Vector pretasks = new Vector();
				while(rs.next()) 
					pretasks.addElement(new Integer(rs.getInt(1)));
				
				rs = state.executeQuery("select TASK_ID from T_TASK_DEPENDENCY where DEPENDENT_TASK_ID = "+this.taskID+" and CONDITION_TYPE = \"POST\"");
				while(rs.next())
					pretasks.addElement(new Integer(rs.getInt(1)));
				
				this.setPreTasks(pretasks);
				
				//Get all posttasks
				rs = state.executeQuery("select DEPENDENT_TASK_ID from T_TASK_DEPENDENCY where TASK_ID = "+this.taskID+" and CONDITION_TYPE = \"POST\"");
				Vector posttasks = new Vector();
				while(rs.next()) 
					posttasks.addElement(new Integer(rs.getInt(1)));
				
				rs = state.executeQuery("select TASK_ID from T_TASK_DEPENDENCY where DEPENDENT_TASK_ID = "+this.taskID+" and CONDITION_TYPE = \"PRE\"");
				while(rs.next())
					posttasks.addElement(new Integer(rs.getInt(1)));
				
				this.setPostTasks(posttasks);
				
				//Get all role dependencies
				rs = state.executeQuery("select ROLE_ID from T_DEPENDENCIES where TASK_ID = "+this.taskID);
				Vector roleDeps = new Vector();
				while(rs.next())
					roleDeps.addElement(new Integer(rs.getInt(1)));
				this.setRoleDependencies(roleDeps);
					
				
				//Get all subtasks
				if(this.composite.equals("Yes")) {
					Vector subs = new Vector();
					rs = state.executeQuery("select DEPENDENT_TASK_ID from T_TASK_DEPENDENCY where TASK_ID = "+this.taskID+" and CONDITION_TYPE = \"COMPOSITE\"");
					while(rs.next()) {
						subs.addElement(new Integer(rs.getInt(1)));
					}
					this.setSubTasks(subs);
				}
			}
   		} catch(SQLException sqle) {
	    	errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("sql.exception"));
   		}		
	 	
	}
	
		
	/**
	 * Gets the tasks with a known responsible role id from the database and sets the properties of the tasks. 
	 * This method returns a Vector of TaskBean objects.
	 *
	 * @param conn A connection with the database
	 * @param errors An ActionErrors object to store any occurring errors in
	 * @param roleID An int representing the ID of the role responsible for this task
	 * @return Vector A vector of TaskBean objects that has the same role as the responsible
	 */
	 public Vector getTasksWithRoleFromDB(Connection conn, ActionErrors errors, int roleID) {
	 
	 	Vector roleTasks = new Vector();
	 	try {
			Statement stmt = conn.createStatement();
			ResultSet rs = stmt.executeQuery("select TASK_ID from T_TASK where ROLE_ID = "+roleID);
									
			while(rs.next()) {
				TaskBean task = new TaskBean();
				int task_id = rs.getInt(1);
				task.getTaskFromDB(conn, errors, task_id);
				roleTasks.addElement(task);
			}
			
   		} catch(SQLException sqle) {
	    	errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("database.could.not.connect"));
   		}		
	 	return roleTasks;
	}
	
	/**
	 * Removes this task permanently from the database.
	 *
	 * @param conn A connection with the database
	 * @param errors An ActionErrors object to store any occurring errors in
	 * @param task_id The id of the task to be deleted from the database
	 */
	public void removeTaskFromDB(Connection conn, ActionErrors errors, int task_id) {
	
		try {
			Statement stmt = conn.createStatement();
			
			//Delete task from T_TASK_CHAR
			stmt.executeUpdate("delete from T_TASK_CHAR where TASK_ID = "+task_id);
			
			//Delete task from T_TASK_DEPENDENCY
			stmt.executeUpdate("delete from T_TASK_DEPENDENCY where TASK_ID = "+task_id);
			
			//Delete task from T_TASK_DEPENDENCY
			stmt.executeUpdate("delete from T_TASK_DEPENDENCY where DEPENDENT_TASK_ID = "+task_id);
			
			//Delete from T_DEPENDENCIES
			stmt.executeUpdate("delete from T_DEPENDENCIES where TASK_ID = "+task_id);
			
			//Delete task from T_TASK
			int rows = stmt.executeUpdate("delete from T_TASK where TASK_ID = "+task_id);
			
			if(rows != 1)
				errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("error.database.delete"));
			
		} catch(SQLException sqle) {
		errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("sql.exception"));
   		}	
		
	}
	
	public int getValueOfCharacteristic(int id) {
		Integer val;
		val = (Integer)characterisation.get(new Integer(id));
		if(val == null)
			return 1;
		return val.intValue();
	}

}
