//Source file: C:\\Program Files\\Apache Group\\Tomcat 4.1\\webapps\\mwcf\\WEB-INF\\classes\\com\\mowahs\\mwcf\\framework\\UserBean.java

package com.mowahs.mwcf.framework;

import java.io.Serializable;
import java.util.Vector;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionError;
import java.sql.*;


/**
 * This class represents a user of the MWCF system, and handles all logic tied to user creation,
 * storage and deletion. This class also handles the access rights of a user.
 * @author Siv Oma Rogdar
 */
public class UserBean implements Serializable {
	
	private String email;
	private String name;
	private String accessLevel;
	private String accessLevelName;
	private String password;
	//The vector of all UserBean objects
	private Vector users;
	// The user ID of this user taken from the database.
	private int userID;
	//A Vector of ScenarioBeans representing the scenarios belonging to this user.
	private Vector scenarios;
		
	
	public UserBean() 
	{
		scenarios = null;
	}
	
	/**
	 * Return the user ID
	 * @return int
	 */
	public int getUserID() 
	{
		return this.userID;		
	}
	
	/**
	 * Set the user ID of this user.
	 * @param userID The id of this user in the database
	 */
	public void setUserID(int userID) 
	{
		this.userID = userID;		
	}
	
	/**
	 * Get the email of this user.
	 * @return java.lang.String
	 */
	public String getEmail() 
	{
    	return this.email;		
	}
	
	/**
	 * Set the email of this user.
	 * @param email The email of this user
	 */
	public void setEmail(String email) 
	{
    	this.email = email;		
	}
	
		
	/**
	 * Get the name of this user.
	 * @return java.lang.String
	 */
	public String getName() {
    	return this.name;		
	}
	
	/**
	 * Set the name of this user.
	 * @param name The name of this user
	 */
	public void setName(String name) 
	{
    	this.name = name;		
	}
		
	/**
	 * Gets the access level of this user.
	 * @return Either (1) "a" for administrator, or (2) "u" for user
	 */
	public String getAccessLevel() {
    	return this.accessLevel;		
	}
	
	/**
	 * Sets the access level of this user.
	 * @param accessLevel Can either be "a" for administrator or "u" for user
	 */
	public void setAccessLevel(String level) {
    	this.accessLevel = level;		
	}
	
	/**
	 * Gets the name for the access level, for example for access level for a, the name is Administrator
	 * @return java.lang.String
	 */
	public String getAccessLevelName() {
	    if (this.accessLevel.equals("a"))
			this.accessLevelName = "Administrator";
		else if (this.accessLevel.equals("u"))
			this.accessLevelName = "User";
				
		return this.accessLevelName;		
	}
	
	/**
	 * Gets the password of this user
	 * @return java.lang.String
	 */
	public String getPassword() {
    	return this.password;		
	}
	
	/**
	 * Sets the password of this user
	 * @param password
	 */
	public void setPassword(String password) {
    	this.password = password;		
	}
	
	/**
	 * Return a Vector of ScenarioBean objects representing the scenarios created by this user.
	 * @return java.util.Vector
	 */
	public Vector getScenarios() {
	 	return this.scenarios;
	}
	
	/**
	 * Set the scenarios attribute of this user.
	 * @param scenarios - The scenarios of this user.
	 */
	public void setScenarios(Vector scenarios) {
		this.scenarios = scenarios;
	}
	
	/**
	 * Add a ScenarioBean object to the Vector of scenarios of this user.
	 * @param scenario - The ScenarioBean object to add to the Vector of scenarios of this user.
	 */
	public void addScenario(ScenarioBean scenario) 
	{
		if(this.scenarios != null)
			this.scenarios.addElement(scenario);
		else{
			this.scenarios = new Vector();
			this.scenarios.addElement(scenario);		
		}
	}
	
	/**
	 * Set the Vector of UserBean objects
	 */
	public void setUsers(Vector users) {
		this.users = users;
	}
	
	/**
	 * Get the vector of UserBean objects
	 */
	public Vector getUsers() {
		return this.users;
	}
	
	
	/**
	 * Checks whether the email is already registered in the database.
	 *
	 * @param conn A connection with the database
	 * @param errors An ActionErrors object to store any occurring errors in 
	 * @param email
	 * @return False if the email is not unique, true if it is unique and the user may go ahead and register this email.
	 */
	public boolean isEmailUnique(Connection conn, ActionErrors errors, String email) 
	{
		boolean bool = false;
   		try {
			Statement stmt = conn.createStatement();
			ResultSet rs = stmt.executeQuery("select USER_ID from T_USER where email = \""+email+"\"");
	       	        
	        if(rs.next())
	        	bool = false;
	        else
	        	bool = true;
	        	
   		} catch(SQLException sqle) {
	    	errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("error.database"));
   		}
   		return bool;		
	}
	
	/**
	 * Gets all users stored in the database. Stores the users in a Vector on this user. 
	 * The users may be retrieved by the getUsers()-method
	 *
	 * @param conn A connection with the database
	 * @param errors An ActionErrors object to store any occurring errors in  
	 */
	public void getAllUsersFromDB(Connection conn, ActionErrors errors) 
	{
		try {
			Statement stmt = conn.createStatement();
			ResultSet rs = stmt.executeQuery("select * from T_USER order by EMAIL");
				
			Vector users = new Vector();					
			while(rs.next()) {
				UserBean user = new UserBean();
				user.setUserID(rs.getInt("USER_ID"));
				user.setEmail(rs.getString("EMAIL"));
				user.setName(rs.getString("NAME"));
				user.setAccessLevel(rs.getString("ACCESSLEVEL"));
				users.add(user);
			}
			this.setUsers(users);
				       	
        } catch(SQLException sqle) {
	    	errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("error.database"));
   		}	
	}
	
	/**
	 * Check with the database whether the user is logging in with a valid email and 
	 * password.
	 * 
	 * @param anEmail The email the user entered
	 * @param aPassword The password entered
	 * @param conn A connection with the database
	 * @return boolean
	 */
	public boolean validateUser(Connection conn, ActionErrors errors, String anEmail, String aPassword) 
	{
		boolean valid = false;
   		try {
	        Statement stmt = conn.createStatement();
	        ResultSet rs = stmt.executeQuery("select USER_ID, NAME, ACCESSLEVEL from T_USER where EMAIL = \""+anEmail+"\" and PASSWORD = \""+aPassword+"\"");
            if(rs.next()){
        	
	        	this.setUserID(rs.getInt(1));
	        	this.setName(rs.getString(2));
	        	this.setAccessLevel(rs.getString(3));
	        	this.setEmail(anEmail);
	        	this.setPassword(aPassword);
	        	valid = true;
		    } 
       		
        } catch (SQLException sqle) {
            errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("error.database"));
        	valid = false;
	 	}
            
    	return valid;		
	}
	
	/**
	 * Saves this user in the database.
	 *
	 * @param conn A connection with the database
	 * @param errors An ActionErrors object to store any occurring errors in 
	 */
	public void saveUserInDB(Connection conn, ActionErrors errors) 
	{
   		try {
   			Statement stmt = conn.createStatement();
   			int rows = stmt.executeUpdate("insert into T_USER (EMAIL, PASSWORD, NAME, ACCESSLEVEL) values (\""+this.email+"\", \""+this.password+"\", \""+this.name+"\", \""+this.accessLevel+"\")");
	       	
	       	if(rows == 1) {
	       		ResultSet rs = stmt.executeQuery("select LAST_INSERT_ID() as lastid");
	       		if(rs.next())
        	    	this.setUserID(rs.getInt(1));
        	    else
        			errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("user.not.saved"));
	       	} else
	       		errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("user.not.saved"));     	
	        
   		} catch(SQLException sqle) {
	    	errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("error.database"));
   		}		
	}
	
	/**
	 * Removes the user permanently from the database. 
	 * The method does also remove all scenarios this user has created 
	 * (including the roles, tasks, and characterisations).
	 * @param conn A connection with the database
	 * @param errors An ActionErrors object to store any occurring errors in 
	 * @param id The id of the user to be deleted
	 */
	public void removeUserFromDB(Connection conn, ActionErrors errors, int id) 
	{
		try {
   			Statement stmt = conn.createStatement();
			ResultSet rs = stmt.executeQuery("select SCENARIO_ID from T_SCENARIO where USER_ID = "+id);
			ScenarioBean scenario = new ScenarioBean();
			
			while(rs.next())
				scenario.removeScenarioFromDB(conn, errors, rs.getInt(1));
				
			int i = stmt.executeUpdate("delete from T_USER where USER_ID = "+id);
			if(i != 1) {
				rs = stmt.executeQuery("select * from T_USER where USER_ID = "+id);
				if(rs.next())
					errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("error.database.delete"));
			}											     
        } catch(SQLException sqle) {
	    	errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("error.database"));
   		}	
	}
	
	/**
	 * Gets a user from the database.
	 *
	 * @param id The id of the user in the database
	 * @param conn A connection with the database
	 * @param errors An ActionErrors object to store any occurring errors in 
	 */
	public void getUserFromDB(Connection conn, ActionErrors errors, int id) 
	{
		try {
   			Statement stmt = conn.createStatement();
			ResultSet rs = stmt.executeQuery("select EMAIL, PASSWORD, NAME, ACCESSLEVEL from T_USER where USER_ID = "+id);
			
			if(rs.next()) {
				this.setEmail(rs.getString(1));
				this.setPassword(rs.getString(2));
				this.setName(rs.getString(3));
				this.setAccessLevel(rs.getString(4));
			}
   			this.setUserID(id);   
   			
        } catch(SQLException sqle) {
	    	errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("error.database"));
   		}	
	}
	
	/**
	 * Updates the data stored on this user in the database
	 * The properties that are updated are set on this user after the database update.
	 *
	 * @param conn A connection with the database
	 * @param errors An ActionErrors object to store any occurring errors in 
	 * @param userId The id of the user to be updated
	 * @param email The new email of the user
	 * @param password The new password
	 * @param name The new name
	 * @param accessLevel The new accessLevel can be either 'a' (for administrator) or 'u' (for user)
	 */
	public void updateUserInDB(Connection conn, ActionErrors errors, int userId, String email, String password, String name, String accessLevel) 
	{
		try {
			Statement stmt = conn.createStatement();
			int rows = stmt.executeUpdate("update T_USER set EMAIL = '"+email+"', PASSWORD = '"+password+"', NAME = '"+name+"', ACCESSLEVEL = '"+accessLevel+"' where USER_ID = "+userId);
			
			if(rows == 1) {
				this.setUserID(userId);
				this.setEmail(email);
				this.setPassword(password);
				this.setName(name);
				this.setAccessLevel(accessLevel);	
			} else 
				errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("error.database"));
			
        } catch(SQLException sqle) {
	    	errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("error.database"));
   		}	
	}
	
	/**
	 * Gets the password from the database with a user's email 
	 *
	 * @param email The registered email of this user
	 * @param conn A connection with the database
	 * @param errors An ActionErrors object to put any occurring errors in
	 * @return The password of the user as a String 
	 */
	public String getPasswordFromDB(Connection conn, ActionErrors errors, String email) {
		
		String password = "";
		try {
			Statement stmt = conn.createStatement();
			ResultSet rs = stmt.executeQuery("select PASSWORD from T_USER where EMAIL = \""+email+"\"");
				
			if(rs.next())
				password = rs.getString(1);
			else 
				errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("error.not.registered"));
			
		} catch(SQLException sqle) {
	    	errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("error.database"));
   		}	
		
		return password;
	}
	
	/**
	 * Send an email with the password of a user to the user from The MWCF web administration
	 *
	 */
	public void sendEmail() {
		String mailFrom = "The MWCF web administration";
			
		
		
	}
	
}

