package com.mowahs.mwcf.site.forms;

import javax.servlet.http.HttpServletRequest;
import org.apache.struts.action.ActionError;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionMapping;
import java.util.Vector;
import java.lang.reflect.Array;

public final class CreateScenarioForm extends ActionForm {

// --------------------------------------------------- Instance Variables

    /**
     * The name of the scenario.
     */
    private String name = null;

    /**
     * The description.
     */
    private String description = null;

	/**
     * A vector of Strings representing the names of the roles of this scenario.
     */
    private Vector roles = null;
    
    /**
     * One of the roles of the scenario.
     */
    private String role = null;
    
    /**
     * The name of the task.
     */
    private String taskName = null;

    /**
     * The description of the task.
     */
    private String taskDescription = null;
    
    /**
     * The responsible of the task.
     */
    private String responsible = null;
    
   	/**
     * A location precondition of the task
     */
    private String preLocation = null;
   
   	/**
     * A resource precondition of the task
     */
    private String preResources = null;
   
   	/**
     * A location postcondition of the task
     */
    private String postLocation = null;
   
   	/**
     * A resource postcondition of the task
     */
    private String postResources = null;
    
    /**
     * The dependency roles of the task
     */
    private String[] dependencyRoles = null;
    
    /**
     * The composite property of a task
     */
	private String composite;
    
    /**
     * The value of the submit button.
     */
	private String submit = null;
	
	/**
     * The step of the wizard
     */
	private int step = 1;
   
   	/**
     * The tasks of the scenario.
     */
	private Vector tasks = null;
   
   	/**
     * The attribute telling whether a Vector of roles has been initialized or not. 
     * Sent as a hidden form value.
     */
    private int noRoles = 1;
    
    //The id of the scenario
    private int scenarioID = -1;
    
   
	// ----------------------------------------------------------- Constructor
	
	public CreateScenarioForm() {
		roles = new Vector();
	}
	

    // ----------------------------------------------------------- Properties


    /**
     * Return the name of the scenario.
     */
    public String getName() {
		return (this.name);
    }


    /**
     * Set the name.
     *
     * @param name The name of the scenario
     */
    public void setName(String name) {
		this.name = name;
    }


    /**
     * Return the description.
     */
    public String getDescription() {
		return (this.description);
    }


    /**
     * Set the description.
     *
     * @param description The description of the scenario
     */
    public void setDescription(String description) {
        this.description = description;
    }

	/**
     * Return the role.
     */
    public String getRole() {
		return (this.role);
    }


    /**
     * Set the role.
     *
     * @param role The role to be added to the scenario.
     */
    public void setRole(String role) {
        this.role = role;
    }
	
	/**
     * Add a role.
     *
     * @param role The role to be added to the roles of the scenario.
     */
    public void addRole(String role) {
        this.roles.add(role);
    }

	/**
     * Return the Vector of roles.
     */
    public Vector getRoles() {
		return (this.roles);
    }


    /**
     * Set the roles.
     *
     * @param roles The Vector of roles of the scenarios.
     */
    public void setRoles(Vector roles) {
        this.roles = roles;
    }


    /**
     * Return the name of the task.
     */
    public String getTaskName() {
		return (this.taskName);
    }

	/**
	 * Set the Vector of tasks of this scenario.
	 *
	 * @param tasks The Vector of task names
	 */
	public void setTasks(Vector tasks) {
		this.tasks = tasks;	
	}
	 
	/**
	 * Get the Vector of tasks of this scenario.
	 */
	public Vector getTasks() {
		return this.tasks;	
	} 

    /**
     * Set the name of the task.
     *
     * @param taskName The name of the task
     */
    public void setTaskName(String taskName) {
        this.taskName = taskName;
    }


    /**
     * Return the description of the task.
     */
    public String getTaskDescription() {
		return (this.taskDescription);
    }


    /**
     * Set the description of the task.
     *
     * @param taskDescription The description of the task
     */
    public void setTaskDescription(String taskDescription) {
        this.taskDescription = taskDescription;
    }

	
	/**
     * Return the responsible of the task.
     */
    public String getResponsible() {
		return (this.responsible);
    }

	/**
     * Set the responsible of the task.
     *
     * @param responsible The responsible of the task
     */
    public void setResponsible(String responsible) {
        this.responsible = responsible;
    }

	
    /**
     * Return any location precondition of the task.
     */
    public String getPreLocation() {
		return (this.preLocation);
    }


    /**
     * Set the location precondition of the task.
     *
     * @param preLocation The location precondition of the task
     */
    public void setPreLocation(String preLocation) {
        this.preLocation = preLocation;
    }
	
	
	 /**
     * Return any resource preconditions of the task.
     */
    public String getPreResources() {
		return (this.preResources);
    }

    /**
     * Set the resource preconditions of the task.
     *
     * @param preResources The resources precondition of the task
     */
    public void setPreResources(String preResources) {
        this.preResources = preResources;
    }
	
	
		
    /**
     * Return any location postcondition of the task.
     */
    public String getPostLocation() {
		return (this.postLocation);
    }


    /**
     * Set the location postcondition of the task.
     *
     * @param postLocation The location postcondition of the task
     */
    public void setPostLocation(String postLocation) {
        this.postLocation = postLocation;
    }
	
	
	/**
     * Return any resource postconditions of the task.
     */
    public String getPostResources() {
		return (this.postResources);
    }

    /**
     * Set the resource postconditions of the task.
     *
     * @param postResources The resources postcondition of the task
     */
    public void setPostResources(String postResources) {
        this.postResources = postResources;
    }
	
	
	
	/**
     * Set the roles the task has as dependencies.
     *
     * @param dependencyRoles The roles the task has as dependencies
     */
    public void setDependencyRoles(String[] dependencyRoles) {
        this.dependencyRoles = dependencyRoles;
    }
    	
	/**
     * Return any dependency roles of the task
     */
    public String[] getDependencyRoles() {
		return (this.dependencyRoles);
    }
	
		
	
	/**
     * Return the composite attribute of the task.
     */
    public String getComposite() {
		return (this.composite);
    }

    /**
     * Set the resource postconditions of the task.
     *
     * @param postResources The resources postcondition of the task
     */
    public void setComposite(String composite) {
        this.composite = composite;
    }
	
		
	public String getSubmit() {
		return submit;
	}
	
	public void setSubmit(String submit){
		this.submit = submit;
	} 
	
	/**
     * Return the number of the wizard process
     */
    public int getStep() {
		return (this.step);
    }
    
     /**
     * Set the step of the wizard.
     *
     * @param step The step of the wizard of creating a scenario
     */
    public void setStep(int step) {
        this.step = step;
    }
	
	
	/**
     * Return the hidden attribute noRoles.
     */
    public int getNoRoles() {
		return (this.noRoles);
    }

	/**
     * Set the noRoles hidden attribute.
     */
    public void setNoRoles(int noRoles) {
		this.noRoles = noRoles	;
    }

	/**
	 * Gets the id of the scenario (included in jsp page: edit_this_scenario.jsp as hidden variable)
	 */
	public int getScenarioID() {
		return this.scenarioID;
	}
	
	/**
	 * Sets the id of the scenario (used in edit_this_scenario.jsp)
	 * @param ID The id of the scenario
	 */
	public void setScenarioID(int ID) {
		this.scenarioID = ID;
	}
	

    /**
     * Reset all properties to their default values.
     *
     * @param mapping The mapping used to select this instance
     * @param request The servlet request we are processing
 	*/   
    public void doreset(ActionMapping mapping, HttpServletRequest request) {

      	this.name = null;
       	this.description = null;
       	this.role = null;
       	this.taskName = null;
       	this.taskDescription = null;
       	this.preLocation = null;
       	this.preResources = null;
       	this.postLocation = null;
       	this.postResources = null;
       	this.composite = null;
       	this.submit = null;
       	this.step = 1;
       	this.roles = null;
       	this.noRoles = 1;
        
    }
    
   

    /**
     * Validate the properties that have been set from this HTTP request,
     * and return an <code>ActionErrors</code> object that encapsulates any
     * validation errors that have been found.  If no errors are found, return
     * <code>null</code> or an <code>ActionErrors</code> object with no
     * recorded error messages.
     *
     * @param mapping The mapping used to select this instance
     * @param request The servlet request we are processing
     */
    
    public ActionErrors validate(ActionMapping mapping, HttpServletRequest request) {

   		ActionErrors errors = new ActionErrors();
   		if(this.submit != null) {
	        if (this.submit.equals("Next") && ((this.name == null) || (this.name.length() < 1)))
	            errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("error.scenario.name.required"));
	        if (this.submit.equals("Add") && ((this.role == "") || (this.role.length() < 1)))
	    	 	errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("roleName.is.empty"));
	    	if (this.submit.equals("Next") && (this.step == 2) && this.roles.isEmpty())
	    		errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("role.required"));
	    	if (this.submit.equals("Next") && (this.step == 3) && (this.taskName == null || this.taskName.length()<1))
	    		errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("taskName.required"));
	    	if (this.submit.equals("Next") && (this.step == 3) && (responsible.equals("notselected")))
	    		errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("responsible.required"));
	    	if (this.submit.equals("Add")) {
	    		for(int i=0; i<this.roles.size(); i++) {
	    			if(this.role.equals(roles.elementAt(i)))
	    				errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("rolename.already.added"));
	    		}
	    	}
	    	if (this.submit.equals("Finish")) {
	    		if(this.dependencyRoles != null && this.dependencyRoles.length > 0) {
		    		for(int i=0; i< this.dependencyRoles.length; i++) {
		    			if(this.responsible.equals(dependencyRoles[i]))
		    				errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("role.not.responsible.and.dependency"));
		    		}
		    	} 		    		
	    	}
	    }	
        return errors;
   	}


}